#ifndef _DLANGLE_H
#define _DLANGLE_H

/**
 * \brief Representation of an angle in a DLImage.
 *
 * A representation of an angle measured from the horizontal (positive
 * <i>x</i>-axis) in the counterclockwise direction.
 *
 */
class DLAngle
{
   friend DLAngle operator+ (const DLAngle & p1, const DLAngle & p2);
   friend DLAngle operator- (const DLAngle & p1, const DLAngle & p2);

   public:

   /**
    * Default DLAngle constructor; creates an angle of 0 radians.
    */
   DLAngle()
      : angle(0.0), specialType(DL_ARBITRARY)
      { ; }

   /**
    * DLAngle copy constructor.
    * \param orig DLAngle to copy
    */
   DLAngle(const DLAngle & orig) { *this = orig; }

   /**
    * Assignment operator.
    * \param right DLAngle to copy
    */
   const DLAngle & operator=(const DLAngle & right)
      {
         this->angle = right.angle;
         this->specialType = right.specialType;
         return *this;
      }

   /**
    * DLAngle destructor.
    */
   virtual ~DLAngle() { ; }

   /**
    * Constructor:
    * Creates a DLAngle from a value in radians.
    */
   DLAngle(double radians);

   /**
    * A selection of possible units of measure for an angle.
    */
   enum DLAngleUnits {
      DL_DEGREES,
      DL_RADIANS,
      DL_GRADS
   };

   /**
    * Some commonly-used specific values for angles.  Some algorithms
    * may check for this attribute of an inputted angle to simplify
    * their calculations.
    *
    * A DLAngle is not marked as special unless specifically
    * constructed that way; therefore, only the first of these
    * otherwise equivalent DLAngles is "special":
    * \code
    * DLAngle one35a(DLAngle::DL_135_DEGREE_ANGLE);
    * DLAngle one35b(135, DLAngle::DL_DEGREES);
    * \endcode
    *
    * In general, the output of operations (such as +, -, += and -=)
    * on special DLAngles will no longer be special. 
    * In the following example,
    * \code
    * DLAngle right(DLAngle::DL_RIGHT_ANGLE);
    * DLAngle straight = right + right;
    * \endcode
    * even though the sum of two right angles is a straight angle,
    * the angle \c straight will not be marked with a DLSpecialAngleType.
    */
   enum DLSpecialAngleType {
      /**
       * An arbitrary angle; indicates that no special processing of
       * this angle will take place, and that processing will proceed
       * based on the angle value alone.
       */
      DL_ARBITRARY,

      /** An angle of 0 degrees. **/
      DL_ZERO_ANGLE,

      /** A 45-degree angle / \f$ \pi/4 \f$ radians. **/
      DL_45_DEGREE_ANGLE,

      /** A 90-degree / \f$ \pi/2 \f$-radian angle. **/
      DL_RIGHT_ANGLE,

      /** A 135-degree / \f$ 3\pi / 4 \f$-radian angle. **/
      DL_135_DEGREE_ANGLE,

      /** A 180-degree, or \f$ \pi \f$-radian angle. **/
      DL_STRAIGHT_ANGLE
   };

   /**
    * The four quadrants of a Cartesian coordinate system.
    */
   enum DLQuadrant {
      /** 
       * An angle for which the quadrant is not defined; i.e. an angle
       * lying on one of the coordinate axes; i.e. an angle of exactly
       * 0, 90, 180 or 270 degrees.
       */
      DL_QUADRANT_UNDEFINED,

      /** An angle between 0 and 90 degrees. **/
      DL_QUADRANT_I,

      /** An angle between 90 and 180 degrees. **/
      DL_QUADRANT_II,

      /** An angle between 180 and 270 degrees. **/
      DL_QUADRANT_III,

      /** An angle between 270 and 0/360 degrees. **/
      DL_QUADRANT_IV
   };

   /**
    * Constructor:
    * Creates a DLAngle from a value in any of the typical
    * representations.
    *
    * Examples:
    * \code
    * DLAngle theta1(30, DLAngle::DL_DEGREES);
    * DLAngle theta2(DL_PI_OVER_2, DLAngle::DL_RADIANS);
    * \endcode
    */
   DLAngle(double value, DLAngleUnits units);

   /**
    * Constructor:
    * Creates a DLAngle from the degrees-minutes-seconds representation.
    */
   DLAngle(double degrees, double minutes, double seconds);

   /**
    * Constructor:
    * Creates one of several particular DLAngles.
    *
    * Example:
    * \code
    * DLAngle theta(DLAngle::DL_45_DEGREE_ANGLE);
    * \endcode
    */
   DLAngle(DLSpecialAngleType type);

   /**
    * \return The measure of the angle in radians.
    */
   double dlRadians() const;

   /**
    * \return The measure of the angle in degrees.
    */
   double dlDegrees() const;

   /**
    * \return The measure of the angle in grads.
    */
   double dlGrads() const;

   /**
    * \return
    * \li \c true if the angle is marked as one of the special
    * types in DLAngle::DLSpecialAngleType
    * \li \c false if the angle is of arbitrary measure
    * (DLAngle::DL_ARBITRARY)
    */
   bool dlIsSpecial() const;

   /**
    * \return The special value of an angle; or DLAngle::DL_ARBITRARY otherwise.
    */
   DLSpecialAngleType dlGetSpecialAngleType() const { return specialType; }

   /**
    * \return The quadrant of the angle in the cartesian coordinate system.
    */
   DLQuadrant dlQuadrant() const;

   /**
    * Counterclockwise angle rotation.
    * \param p2 DLAngle to rotate by
    */
   const DLAngle & operator +=(const DLAngle & p2);

   /**
    * Clockwise angle rotation.
    * \param p2 DLAngle to rotate by
    */
   const DLAngle & operator -=(const DLAngle & p2);

   /**
    * Reflection across the <i>x</i>-axis.
    */
   inline const DLAngle operator-() const
      {
         return DLAngle(-this->angle);
      }

   /** \cond **/

   /**
    * Unary + operator.
    */
   inline const DLAngle operator+() const 
      {
         return *this;
      }

   /** \endcond **/

   /**
    * Equality operator (compare two angles)
    * \param p2 DLAngle to compare to
    */
   inline bool operator==(const DLAngle & p2) const
      {
         return (this->angle == p2.angle && this->specialType == p2.specialType);
      }

   /**
    * Conversion to \c double operator.
    *
    * Allows an angle to be used as input to functions that require
    * input of type \c double, for example:
    * \code
    * DLAngle theta;
    * double length;
    * ...
    * double y = length * sin(theta);
    * cout << "angle theta, in radians: " << theta << endl;
    * \endcode
    * \return Measure of the angle in radians.
    */
   inline operator double() const { return angle; }

   /**
    * Convert the DLAngle to an angle in the range \f$ [0,2\pi) \f$ radians.
    */
   void dlReduce();

   private:
   static inline double degrees2radians(double degrees);
   static inline double grads2radians(double grads);
   static inline double radians2degrees(double radians);
   static inline double radians2grads(double radians);

   /// internal storage of the actual angle.
   /// represented in radians.
   double angle;

   /// indicates that this angle is one of several special angle
   DLSpecialAngleType specialType;
};

/// The sum of two DLAngle%s
DLAngle operator+ (const DLAngle & p1, const DLAngle & p2);

/// The difference of two DLAngle%s
DLAngle operator- (const DLAngle & p1, const DLAngle & p2);

#endif
