#ifndef _DLBlur_H_
#define _DLBlur_H_

#include "DLImage.h"
#include "DLPoint.h"
#include "DLBitsPerPixelConverter.h"

#include <cmath>

/**
 * \brief Blur an image.
 */
class DLBlur
{
   public:
   /** \cond **/
   /**
    * Default DLBlur Constructor
    */
   DLBlur();

   /**
    * Default DLBlur Destructor
    */
   ~DLBlur();
   /** \endcond **/

/**
 * Blur an image.
 * This algorithm is implemented as a static function, so no object of
 * class DLBlur needs to be instantiated. See below for a usage example:
 * \code
 * DLImage i ("test.tif");
 * DLImage blurredImage = DLBlur::dlBlur(i);
 * \endcode
 *
 * This function (currently) requires a color image as input.
 * At each application of the blur operation, a 5x5 blur
 * filter is convolved with the image:
 * \f[ \left[
 * \begin{array}{rrrrr}
 * 0 & 0 & 1 & 0 & 0 \\
 * 0 & 8 & 21 & 8 & 0 \\
 * 1 & 21 & 59 & 21 & 1 \\
 * 0 & 8 & 21 & 8 & 0 \\
 * 0 & 0 & 1 & 0 & 0
 * \end{array} \right] \f] 
 * \param in DLImage to blur
 * \param numberOfTimes number of times to apply blur operation
 * \return blurred image
 */	
   static DLImage dlBlur ( const DLImage & in, int numberOfTimes = 1 );

   /**
    * This method provides a mechanism for performing a very fast "blur" of 
    * binary images.  This is done by creating an alternate representation of 
    * the image, refered to as the distance representation.  In the distance
    * representation each pixel is represented by how far it is from its neighbors 
    * horizontally.
    *
    * Once the distance representation has been created, a method is provided to
    * threshold based on a user defined parameter.  A "blurred" image is returned
    * in which all pixels whose value in the distance representation is
    * less then the threshold are set to black.
    *	
    * The current distance representation is constructed such that the values
    * it contains can range from 0 (for a pixel that is black in the original
    * image) to (the image width - 1).
    *
    * This method operates on a 1 byte/pixel ("grayscale") representation of a 
    * binary image.  It is up the the application to assure the input image
    * represents binary data.  All non-white pixels in the input image will 
    * be treated as if they are black in calculating the distanceRepresentation.
    * An exception is thrown if input image is not grayscale.
    *
    * A single DLBlur instance may be used to blur multiple images using the \c reload parameter.
    * In order to preserve the added speed provided by this algorithm, one should do all blurring
    * on one image before reloading with a different image.  In the event that multiple blurs need
    * to occur in parallel, it is suggested that another DLBlur object be created.  See the below
    * example code:
    * \code
    * DLImage a1("bw_8bit.tif");
    * DLImage a2("bw_8bit2.tif");
    *
    * DLBlur b;
    * b.dlBinaryHorizontalBlur(a1,5);
    * b.dlBinaryHorizontalBlur(4);		// note that the image name is
    *					// no longer necessary.
    * b.dlBinaryHorizontalBlur(a1,3);	// but can be given nonetheless.
    *
    * b.dlBinaryHorizontalBlur(a2,5,true);	// note that it is necessary
    *					// to tell dlBinaryHorizontalBlur
    *					// that you want to reload with
    *					// a new image.
    * b.dlBinaryHorizontalBlur(4);		// now blurring a2 with threshold of 4
    * \endcode
    *
    * \param in DLImage to blur ( bw image in 8-bit colorspace)
    * \param thresholdValue int value under which pixels will be set to black
    * \param reload flag informing the method that this is a new image to be blurred
    * \return blurred image
    * \throw DLException DL_WRONG_FORMAT_EXCEPTION Only accepts grayscale (8-bit) BW images
    */
   DLImage dlBinaryHorizontalBlur ( const DLImage & in, int thresholdValue, bool reload = false );

   /**
    * See DLImage DLBlur::dlBinaryHorizontalBlur ( const DLImage & in, int thresholdValue, bool reload = false );
    * \param thresholdValue int value under which pixels will be set to black
    * \return blurred image
    * \throw DLException DL_NULL_POINTER_EXCEPTION when an image has not been loaded yet.
    */
   DLImage dlBinaryHorizontalBlur ( int thresholdValue );


   private:
	
   /** 
    * Takes a DLImage as input.  Memory is allocated
    * for the distance representation of the image and the distance 
    * representation is calculated.
    */
   void dlHorizontalBlurLoad ( const DLImage & inputImage );

   float *distanceRep;

   /** integer array containing a histogram of the number of pixels at each
       level in the distance representation */

   int *distanceRepHistogram;

   /** Input image width */
   int _imageWidth;

   /** Input image height */
   int _imageHeight;

};//end class

#endif //_DLBlur_H_
