#include "DLPoint.h"
#include "DLDatarun.h"
#include "DLImage.h"
#include "DLColor.h"

#include <vector>

using namespace std;

#ifndef _DLCOMPONENT_H_
#define _DLCOMPONENT_H_

/**
 * \brief Connected component object.
 *
 * DLComponent is a data object which stores all the information needed to describe a 
 * connected component. A connected component is a contiguous region of black pixels.
 * Here, contiguous means 8-connected, so that the image below (each pixel represented
 * by an X) contains a single connected component, since the black pixels are adjacent
 * horizontally, vertically, or across a diagonal. 
 * \verbatim
 XX XX
 X X  XX
 XX XX
 \endverbatim
 *
 * The DLConnectedComponents class stores a list of DLComponents
 * calculated from an entire image. An individual DLComponent can be accessed by 
 * getting the vector of DLComponent%s using DLConnectedComponents::dlGetConnectedComponents()
 * or using the various DLConnectedComponents iterators
 * to traverse the DLConnectedComponents object.
 *
 * Examples:
 * \code
 * DLImage i("test.tif"); // load an image
 * DLConnectedComponents cc;
 * cc.dlCreateConnectedComponents(i); // calculate connected components
 * std::vector<DLComponent> cv = cc.dlGetConnectedComponents();
 * cout << "First component's width: " << cv[0].dlGetWidth() << endl;
 * DLConnectedComponents::iterator ii = cc.begin();
 * cout << "First component's width: " << ii->dlGetWidth() << endl;
 * ii++; // advance iterator
 * cout << "Second component's width: " << ii->dlGetWidth() << endl;
 * \endcode
 *
 * The various dlGet...() functions are used to get information about an 
 * individual connected component. The various dlSet...() functions can be
 * used to overwrite various attributes with new values, but note that they
 * do not cause recalculation of the other attributes, and do not affect any
 * of the DLComponent's internal information other than that single attribute being set.
 */
class DLComponent
{
   /**
    * Friend to DLConnected Component to allow sharing of private data
    */
   friend class DLConnectedComponents;
  
   public:

   /**
    * Default DLComponent constructor
    */
   DLComponent();
	
   /**
    * Default DLComponent copy constructor
    * \param orig orginal image to be copied
    */
   DLComponent( const DLComponent & orig);

   /**
    * Default DLComponent destructor
    */
   virtual ~DLComponent(); 

   /**
    * Get the width of the component
    * \return width
    */
   int dlGetWidth()  const  { return cmax-cmin + 1;}

   /**
    * Get the height of the component
    * \return height
    */
   int dlGetHeight()  const  { return rmax-rmin + 1;} 

   /**
    * Get the area of the component: the mass is the number of dark pixels
    * in the connected component (previously called "area")
    * \return mass
    */
   int dlGetMass()  const  {return area;}

   /**
    * Gets the row centroid of the component.
    * The coordinate returned is relative to the original image's origin.
    *
    * \return row centroid 
    */
   float dlGetRowCentroid()  const  {return rcentroid;}                  

   /**
    * Get the column centroid of the component.
    * The coordinate returned is relative to the original image's origin.
    *
    * \return column centroid
    */
   float dlGetColCentroid()  const  {return ccentroid;}                 

   /**
    * Get the length of the perimeter of the component
    * \return length of perimeter
    */
   int dlGetPerimeter()  const  {return perimeter;}             

   /**
    * Get the bounding box extents: uppermost row of the component.
    * The coordinate returned is relative to the original image's origin.
    *
    * \return uppermost row
    */
   int dlGetRowMin()  const  {return rmin;}           

   /**
    * Get the bounding box extents: lowermost row of the component.
    * The coordinate returned is relative to the original image's origin.
    *
    * \return lowermost row
    */
   int dlGetRowMax()  const  {return rmax;}                

   /**
    * Get the bounding box extents: leftmost column of the component.
    * The coordinate returned is relative to the original image's origin.
    *
    * \return leftmost column
    */
   int dlGetColMin()  const  {return cmin;}                      

   /**
    * Get the bounding box extents: rightmost column of the component.
    * The coordinate returned is relative to the original image's origin.
    *
    * \return int cmax
    */
   int dlGetColMax()  const  {return cmax;}
 
   /**
    * Get the  row index of seed pixel of the component.
    * The seed pixel is the pixel which forms the core of the
    * connected component; other pixels in the component were
    * discovered through their adjacency to the seed pixel.
    * The coordinate returned is relative to the original image's origin.
    * \return seed pixel row index
    */
   int dlGetRowSeed()  const  {return rseed;}
 
   /**
    * Get the column index of the seed pixel of the component.
    * The seed pixel is the pixel which forms the core of the
    * connected component; other pixels in the component were
    * discovered through their adjacency to the seed pixel.
    * The coordinate returned is relative to the original image's origin.
    * \return seed pixel column index
    */
   int dlGetColSeed()  const  {return cseed;}                     

   /**
    * Get the number of holes of the component.
    * A hole is an area of 4-connected white pixels entirely within
    * the interior of a connected component.
    * 
    * The following connected component has one hole:
    * \verbatim
    XXXXXXXXXXXX
    XXXXXX   XXX
    XXXXXXX  XXX
    XXXXX    XXX
    XXXXXXXXX XX
    XXXX      XX
    XX         X
    \endverbatim
    * The following connected component has two holes:
    * \verbatim
    XXXXXX
    X  XXX
    XXX  X
    XXXXXX
    \endverbatim
    * \return number of holes
    */
   int dlGetNumHoles()  const  {return number_of_holes;}

   /**
    * Get the number of runs in the component.
    * A run is a contiguous horizontal row of black pixels.
    * So the following connected component has 6 runs:
    * \verbatim
    XXXX  XXXXXX   XXXX
    XXXXXXXXXXXXXXXX
    XXXXXX   XXXXX
    \endverbatim
    * \return number of runs
    */
   int dlGetNumRuns()  const  {return num_runs;}                

   /**
    * Get the vector of the component's runs.
    * A run is a contiguous horizontal row of black pixels within the component.
    * \return vector of runs
    */
   vector <DLDatarun> dlGetRuns()  const  {return runs;}

   /**
    * Get the number of up-concavities in the component.
    * The following connected component has three up-concavities:
    * \verbatim
    X     XXXXXX   XXXX
    XX   XXXXXXXXXXXXXX
    XXX XXXXXXXXXXXXXXX
    XXXXXXXX   XXXXXXXX
    XXXXXXXXXXXXXXXXXXX
    \endverbatim
    * \return number of up-concavities
    */
   int dlGetNumConcavitiesUp()  const  {return num_conc_up;}                 

   /**
    * Get the vector of the component's up-concavities.
    * The DLDatarun representation of the up-concavity
    * will indicate the run of white pixels farthest down in 
    * the concavity.
    * \return vector of up-concavities 
    */
   vector <DLDatarun> dlGetConcavitiesUp()  const  {return conc_up;}

   /**
    * Get the number of down-concavities in the component.
    * The following connected component has three down-concavities:
    * \verbatim
    XXXXXXXXXXXXXXXXXXX
    XXXXXXXX   XXXXXXXX
    XXX XXXXXXXXXXXXXXX
    XX   XXXXXXXXXXXXXX
    X     XXXXXX   XXXX
    \endverbatim
    * \return number of down-concavities
    */
   int dlGetNumConcavitiesDown()  const  {return num_conc_down;}              

   /**
    * Get the vector of the component's down-concavities.
    * The DLDatarun representation of the down-concavity
    * will indicate the run of white pixels farthest up in 
    * the concavity.
    * \return vector of up-concavities 
    */
   vector <DLDatarun> dlGetConcavitiesDown()  const  {return conc_down;}

/** \cond **/
   /**
    * Get the number of points in the convex hull of the component.
    * \return number of points in convex hull
    */
   int dlGetNumConvexHulls()  const  {return num_points_convex_hull;}    

   /**
    * Get the list of convex hulls of the Component
    * \return  vector convexHull
    */
   vector <DLPoint> dlGetConvexHulls()  const  {return convex_hull;}

   /**
    * Get the area of convexhull of the Component
    * \return float areaConvexHall
    */
   float dlGetAreaConvexHulls()  const  {return area_of_convex_hull;} 
	 
   /**
    * Get the perimeter convex hull of the Component
    * \return float perimeter
    */
   float dlGetPerimeterConvexHulls()  const  {return perimeter_of_convex_hull;} 
	 
/** \endcond **/

   /**
    * Get the <i>n</i>th central moment of the component. The moments
    * are defined as follows, where \f$\bar{x}\f$ denotes the
    * component's column (horizontal) centroid, and \f$\bar{y}\f$
    * denotes the component's row (vertical) centroid:
    *
    * <table>
    * <tr><th><i>n</i></th><th>Description</th> <th></th> <th><i>n</i></th><th>Description</th> <th></th>
    *     <th><i>n</i></th><th>Description</th> </tr>
    * <tr><td>0</td> <td>component mass, or \f$|C|\f$</td> </tr>
    * <tr><td>1</td> <td> \f[ \frac{1}{|C|} \sum_{p \in C}^n (p_x - \bar{x}) \approx 0 \f] </td> <td></td>
    *          <td>4</td> <td> \f[ \frac{1}{|C|} \sum_{p \in C}^n (p_x - \bar{x})(p_y - \bar{y}) \f] </td> <td></td>
    *               <td>7</td> <td> \f[ \frac{1}{|C|} \sum_{p \in C}^n (p_x - \bar{x})^2 (p_y - \bar{y}) \f] </td> </tr>
    * <tr><td>2</td> <td> \f[ \frac{1}{|C|} \sum_{p \in C}^n (p_y - \bar{y}) \approx 0 \f] </td> <td></td>
    *          <td>5</td> <td> \f[ \frac{1}{|C|} \sum_{p \in C}^n (p_y - \bar{y})^2 \f] </td> <td></td>
    *               <td>8</td> <td> \f[ \frac{1}{|C|} \sum_{p \in C}^n (p_x - \bar{x}) (p_y - \bar{y})^2 \f] </td> </tr>
    * <tr><td>3</td> <td> \f[ \frac{1}{|C|} \sum_{p \in C}^n (p_y - \bar{y})^2 \f] </td> <td></td>
    *          <td>6</td> <td> \f[ \frac{1}{|C|} \sum_{p \in C}^n (p_x - \bar{x})^3 \f] </td> <td></td>
    *               <td>9</td> <td> \f[ \frac{1}{|C|} \sum_{p \in C}^n (p_y - \bar{y})^3 \f] </td> </tr>
    * </table>
    *
    * \note DLConnectedComponents::dlCalculateMoments() must have
    * already been invoked after calculating the components themselves
    * before using dlGetMoment().
    * \param n index of the moment to return
    * \return component's <i>n</i>th moment
    */
   float dlGetMoment(int n)  const  {return moments[n];}

   /// \name Set-attribute functions
   /// These functions can be
   /// used to overwrite various attributes with new values, but note that they
   /// do not cause recalculation of the other attributes, and do not affect any
   /// of the DLComponent's internal information other than that single attribute being set.
   //@{

   /**
    * Sets the area of the component.
    * \param newArea new area
    */
   void dlSetArea(int newArea){area = newArea;}

   /**
    * Set the row centroid  of the component
    * \param newRcentroid new row centroid 
    */
   void dlSetRowCentriod(float newRcentroid){rcentroid = newRcentroid;}

   /**
    * Set the column centroid of the Component
    * \param newCCentroid int col centroid
    */
   void dlSetColCentroid(float newCCentroid){ccentroid=newCCentroid;}

   /**
    * Set the length of perimeter of the component
    * \param  newPerimeter new length of perimeter
    */
   void dlSetPerimeter(int newPerimeter){perimeter = newPerimeter;}

   /**
    * Set the bounding box extents: uppermost row of the component
    * \param newRmin new uppermost row
    */
   void dlSetRowMin(int newRmin){rmin = newRmin;}

   /**
    * Set the bounding box extents: lowermost row of the component
    * \param newRmax new lowermost row
    */
   void dlSetRowMax(int newRmax){rmax = newRmax;}

   /**
    * Set the bounding box extents: leftmost column of the component
    * \param newCmin new leftmost column
    */
   void dlSetColMin(int newCmin){cmin = newCmin;}

   /**
    * Set the bounding box extents of the Component
    * \param newCmax int
    */
   void dlSetColMax(int newCmax){cmax = newCmax;}

   /**
    * Set the row index of seed pixel of the Component
    * \param newRseed int
    */
   void dlSetRowSeed(int newRseed){rseed = newRseed;}

   /**
    * Set the column index of the seed pixel of the component
    * \param newCseed new seed pixel column index
    */
   void dlSetColSeed(int newCseed){cseed = newCseed;}

   /**
    * Set the number of holes of the component
    * \param holes new number of holes
    */
   void dlSetNumHoles(int holes){number_of_holes = holes;}           


/** \cond **/

   /**
    * Set the number of runs in seg of the Component
    * \param  num int
    */
   void dlSetNumRuns(int num){num_runs = num;}

   /**
    * Add a run to the list of runs of the Component
    * \param run DLDatarun* 
    */
   void dlAddRun(DLDatarun& run){runs.push_back(run);}          

/** \endcond **/

   /**
    * Set the number of up concavities  of the Component
    * \param num int
    */
   void dlSetNumConcavitiesUp(int num){num_conc_up = num;}

   /**
    * Set the number of down concavities of the Component
    * \param num int
    */
   void dlSetNumConcavitiesDown(int num){num_conc_down = num;}

   /**
    * Add an up concavity to the Component
    * \param conc DLDatarun
    */
   void dlAddConcavityUp(DLDatarun conc){conc_up.push_back(conc);}               

   /**
    * Add a down concavity to the component's list of concavities.
    * \param conc DLDatarun
    */
   void dlAddConcavityDown(DLDatarun conc){conc_down.push_back(conc);}            

   /**
    * Set the number convex hull of the Component
    * \param num number of hulls
    */
   void dlSetNumConvexHulls(int num){num_points_convex_hull = num;}

   /**
    * Add a convexHull to the Component
    * \param convexHull DLPoint 
    */
   void dlAddConvexHull(DLPoint convexHull){convex_hull.push_back(convexHull);}              /* points on convex_hull     */

   /**
    * Set the area convex hull of the Component
    * \param convexHull float area
    */
   void dlSetAreaConvexHulls(float convexHull){area_of_convex_hull = convexHull;}

   /**
    * Set the Area of the Component
    * \param perimeterConvexHull float perimeter
    */
   void  dlSetPerimeterConvexHulls(float perimeterConvexHull){perimeter_of_convex_hull = perimeterConvexHull;}


/** \cond **/ 

   /**
    * Set the moment of the Component at index
    * \param idx int
    * \param value fload
    */
   void dlSetMoment(int idx, float value){moments[idx] = value;}

/** \endcond **/

   /**
    * Set the component's color, for use when rendering a 
    * color image of a set of connected components.
    * \param newColor set to this color
    *
    */
   void dlSetComponentColor(const DLColor &newColor){color = newColor;};


   //@}

   /// \name DLImage Rendering Functions
   //@{

   /**
    * Render a grayscale image of the connected component.
    * Note that the component itself contains only black and white pixels,
    * and therefore the grayscale image will have only grayscale values
    * of 0x00 or 0xFF.
    * The image will be the size of the bounding box of the component.
    * \return  binary DLImage of connected component
    */
   DLImage dlGetImage()  const;

    
   /**
    * Return the image of the connected component but set the color
    * of the image with the color value set in the component's color variable.
    * The default is BLACK. The color for an individual DLComponent
    *  can be set by using dlSetComponentColor(). 
    * See DLConnectedComponents::dlSetFilteredCCColor() for a way to
    * set the color of many components at once.
    * The image will be the size of the bounding box of the component.
    * \return color DLImage of connected component
    */
   DLImage dlGetColorImage()  const;

   /**
    * Render a binary image of the connected component.
    * The image will be the size of the bounding box of the component.
    * \return binary DLImage of connected component
    */
   DLImage dlGetBinaryImage( )  const;

   //@}

   /**
    *  Assignment operator
    * \param right DLComponent
    * \return DLComponent copied instance to be returned
    */
   const  DLComponent& operator=( const DLComponent & right );
    
   /**
    * Get component color
    * \return color DLColor
    */
   const DLColor dlGetComponentColor(){return color;};

   private:
   /**
    * num of pixels in region 
    */
   int area;    
   /**
    * row centroid  
    */
   float rcentroid;                  
   /**
    * col centroid 
    */
   float ccentroid;                  
   /**
    * length of perimeter
    */
   int perimeter;                    
   /**
    * bounding box extents
    */
   int rmin ;                        
   /**
    * bounding box extents 
    */
   int rmax ;                        
   /**
    * bounding box extents 
    */
   int cmin ;                       
   /**
    * bounding box extents
    */
   int cmax ;                        
   /**
    * row index of seed pixel
    */
   int rseed;                        
   /**
    * col index of seed pixel
    */
   int cseed ;                      
   /**
    * number of holes in component 
    */
   int number_of_holes;            
   /**
    * number of runs in seg
    */
   int num_runs;                     
   /**
    * list of runs 
    */
   vector <DLDatarun> runs;                 
   /**
    * number of up concavities 
    */
   int num_conc_up;                 
   /**
    * list of up concavitys
    */
   vector <DLDatarun> conc_up;              
   /**
    * number of down concavities 
    */
   int num_conc_down;                
   /**
    * list of down concavitys 
    */
   vector <DLDatarun> conc_down;           
   /**
    * num_points_convex_hull
    */
   int num_points_convex_hull;      
   /**
    * points on convex_hull 
    */
   vector <DLPoint> convex_hull;             
   /**
    * area_of_convex_hull 
    */
   float area_of_convex_hull;        
   /**
    * perimeter_of_convex_hull  
    */
   float perimeter_of_convex_hull;   
   /**
    * first 14 moments 
    */
   float moments[14];                
   /**
    * link to user defined data  TBD
    */
   int *user_data;                  

   /**
    * Color for generating the DLImage
    */ 
   DLColor color;
};


#endif // _DLCOMPONENT_H_
