#ifndef _DLCONNECTEDCOMPONENTS_H_
#define _DLCONNECTEDCOMPONENTS_H_

#include "DLImage.h"
#include "DLComponent.h"
#include "DLConst.h"
#include "DLTypes.h"
//#include "DLCompIterator.h"

#include <vector>
using namespace std;

/**
 * \brief Calculates and stores connected components
 * of an DLImage.  
 * 
 * DLConnectedComponents stores a list of DLComponent%s to represent the connected components
 * of the entire image. An image must be binary (black and white) in order to calculate
 * its connected components. See DLBitsPerPixelConverter for methods to convert color
 * or grayscale images to binary.
 *
 * Typical usage: 
 * \code
 * DLImage i("test.tif"); // load binary (B&W) image
 * DLConnectedComponents cc; // declare connected components object
 * cc.dlCreateConnectedComponents(i); // calculate components from image
 * cout << cc.dlGetNumCCs() << endl; // output number of components in image
 * \endcode 
 *
 * Use the filtering functions to keep or remove components based on a variety of
 * criteria. Filtering does not actually destroy any connected components, and any
 * filtering operations can be undone using dlResetCCFilter(). 
 * Filtering functions can be applied in series to achieve a cumulative effect.
 *
 * Once filtered,
 * use functions such as dlRenderFilteredCCs(), fbegin() and fend()
 * to use the filtered components. 
 * See  DLComponent and the various flavors of component iterators, including
 * DLConnectedComponents::iterator, DLConnectedComponents::fiterator,
 * DLConnectedComponents::const_iterator and DLConnectedComponents::const_fiterator
 * for information about accessing individual
 * connected components.
 * 
 * Example: 
 * \code
 * cc.dlFilterCCBoxGreater(2,2); // remove small components (e.g. speckle)
 * DLImage despeckled = cc.dlRenderFilteredCCs(); // create new image from only large components
 * \endcode
 */

class DLConnectedComponents
{
   public:

   /**
    * Default DLConnectedComponents constructor
    */
   DLConnectedComponents();

   /**
    *  DLConnectedComponents Copy  Constructor
    */
   DLConnectedComponents(const DLConnectedComponents & orig);

   /**
    *  Assignment operator
    * \param right DLConnectedComponents 
    */
   const DLConnectedComponents & operator=(const DLConnectedComponents & right);


   /**
    * Default DLConnectedComponents destructor
    */
   virtual ~DLConnectedComponents();

   /**
    * Calculate connected components for a given image.
    * The result will be saved in the DLConnectedComponents object
    * and can be accessed.
    *
    * \param img binary (B&W) image to process
    */
   void dlCreateConnectedComponents(const DLImage &img);

   /**
    * Gets the list of the calculated connected components. 
    * Note that this makes a copy of the entire internal structure 
    * of DLConnectedComponents.
    * See begin() and end()
    * for an alternative method for accessing individual
    * components which does not require a full copy.
    * 
    * \return vector of DLComponent%s
    */
   vector <DLComponent> dlGetConnectedComponents()  const;

   /** 
    * Calculate the first 10 moments of all connected components.
    * See DLComponent::dlGetMoment() for a description of the moments.
    */
   void dlCalculateMoments();
   
   /**
    * Filtering Connected Components
    * Multiple methods are utilized, one per filtering algorithm.
    * Each filtering method uses filtCompIndices as the input vector,
    * and the result is to modify filtCompIndices.
    */

   /**
    * Get filtered connected components.
    * See the iterator functions fbegin() and fend()
    * for an alternative method for accessing individual
    * components which does not require a full copy.
    * \return the current set of filtered connected components
    */
   vector <DLComponent> dlGetFilteredCCs()  const;


   /**
    * Reset Filter for connected components 
    * All of the components are re-selected, and filtCompIndices
    * (the indices of filtered components) is
    * reinitialized for all components.
    */
   void dlResetCCFilter();

   /// \name Filtering Functions
   //@{

   /**
    * Filter connected components by height: 
    * retain components with height between or equal to the limits.
    * (Exclude too-small and too-large components)
    * \param minHeight minimum height for including component
    * \param maxHeight maximum height for including component
    * \return number of components included
    */
   int dlFilterCCHeightBetween(int minHeight, int maxHeight);

   /**
    * Filter connected components by width:
    * retain components with width between or equal to the limits.
    * (Exclude too-small and too-large components)
    * \param minWidth minimum width for including component
    * \param maxWidth maximum width for including component
    * \return number of components included
    */
   int dlFilterCCWidthBetween(int minWidth, int maxWidth);

   /**
    * Filter connected components by their vertical position
    * in an image.
    * (Exclude above-top and below-bottom components.)
    * This filter retains only components entirely below (or touching) 
    * toprow and above (or touching) bottomrow.
    * \param toprow top limit for including component - Row number
    * \param bottomrow bottom limit for including component - Row number
    * \return number of components included
    */
   int dlFilterCCRemoveTopBottom(int toprow, int bottomrow);

   /**
    * Filter connected components by  their horizontal position
    * in an image.
    * (Exclude far-left and far-right components.)
    * This filter retains only components lying entirely to the right of (or touching)
    * leftcol and entirely to the left of (or touching) rightcol.
    * \param leftcol left limit for including component - Column number
    * \param rightcol right limit for including component - Column number
    * \return number of components included
    */
   int dlFilterCCRemoveLeftRight(int leftcol, int rightcol);

   /**
    * Filter connected components by density.
    * This filter retains only components whose density exceeds or is equal to the given parameter.
    * (Exclude low-density components.)
    * Density is calculated by taking the number of dark pixels in the component
    * and dividing by area of the component's bounding box. 
    * ( Density can be a float in the range (0.0, 1.0]. )
    * \param density minimum density
    * \return number of components included
    */
   int dlFilterCCDensityGreater(float density);

   /**
    * Filter connected components by density.
    * This filter retains only components whose density is below or equal to the given parameter.
    * (Exclude high-density components.)
    * Density is calculated by taking the number of dark pixels in the component
    * and dividing by area of the component's bounding box. 
    * ( Density can be a float in the range (0.0, 1.0]. )
    * \param density maximum density
    * \return number of components included
    */
   int dlFilterCCDensityLess(float density);

   /**
    * Filter connected components by density.
    * This filter retains only components whose density exceeds or is equal to mindensity and
    * is less than or equal to maxdensity.
    * (Exclude too-high- and too-low-density components.)
    * Density is calculated by taking the number of dark pixels in the component
    * and dividing by area of the component's bounding box. 
    * ( Density can be a float in the range (0.0, 1.0]. )
    * \param mindensity minimum density
    * \param maxdensity maximum density
    * \return number of components included
    */
   int dlFilterCCDensityBetween(float mindensity, float maxdensity);

   /**
    * Filter connected components by size of their rectangular bounding box.
    * This filter retains components whose height and width are both smaller (or equal to)
    * than the given parameters.
    * (Exclude larger-boxed components)
    * \param width width float
    * \param height height float
    * \return number of components included
    */
   int dlFilterCCBoxLess(float width, float height);

   /**
    * Filter connected components by size of their rectangular bounding box.
    * This filter retains components whose height and width are both larger (or equal to)
    * than the given parameters.
    * (Exclude smaller-boxed components)
    * \param width width float
    * \param height height float
    * \return number of components included
    */
   int dlFilterCCBoxGreater(float width, float height);

   /**
    * Filter connected components by mass. 
    * This filter retains components whose mass is greater than or equal to the
    * given parameter.
    * (Exclude low-mass components)
    * "Mass" is the number of dark pixels in the connected component.
    * \param mass minimum mass
    * \return number of components included
    */
   int dlFilterCCMassGreater(float mass);

   /**
    * Filter connected components by mass. 
    * This filter retains components whose mass is less than or equal to the
    * given parameter.
    * (Exclude high-mass components)
    * "Mass" is the number of dark pixels in the connected component.
    * \param mass maximum mass
    * \return number of components included
    */
   int dlFilterCCMassLess(float mass);

   /**
    * Filter connected components by mass. 
    * This filter retains components whose mass is between (or equal to) minmass and maxmass. 
    * "Mass" is the number of dark pixels in the connected component.
    * (Exclude too-high- or too-low-mass components)
    * \param minmass minimum mass
    * \param maxmass maximum mass
    * \return number of components included
    */
   int dlFilterCCMassBetween(float minmass, float maxmass); 

   //@}

   /**
    * Render connected components
    * \return image of connected components
    */
   DLImage dlRenderCCs();

   /**
    * Render filtered connected components: form an image of only the components
    * retained after any filter functions have been called. Note that if this 
    * function is called immediately after calculating the connected components, 
    * or after dlResetCCFilter(), it will have the same effect as 
    * dlRenderCCs().
    * \return image of filtered connected components
    */
   DLImage dlRenderFilteredCCs();

   /**
    * Render a color image of the connected components. Use dlSetFilteredCCColor()
    * to color groups of components.
    * \return image of connected components
    */
   DLImage dlRenderColorCCs();

   /**
    * Copy connected components
    * \return connected components
    */
   DLConnectedComponents dlCopyCCs();

   /** \cond **/

   /**
    * set CalConvexHulls
    * \param calculateConvexHulls flag to set
    */
   void setCalConvexHulls(bool calculateConvexHulls);

   /**
    * set gets ConvexHalls
    * \return convexHalls flag
    */
   bool getCalConvexHalls()  const;

   /**
    * set CalMoments
    * \param calculateMoments flag to set
    */
   void setCalMoments(bool calculateMoments);

   /**
    * set gets CalMoments flag
    * \return CalMoments flag
    */
   bool getCalMoments()  const;

   /** \endcond **/

   /**
    * Get the number of connected components
    * \return result
    */
   int dlGetNumCCs() const;

   /**
    * Creates integer vector, the same height and width as the input image.
    * Each entry (x,y) in the array will be either the index of the connected component
    * to which image pixel (x,y) belongs, or '-1' if that pixel location is not 
    * part of any connected component.
    *
    * Note: dlRenderFilteredCCs() must be called before calling
    * dlGetConnectedCCMap()
    *
    * \return component map integer vector
    */
   const vector<int> dlGetConnectedCCMap();

   /**
    * Assign a color value to the current filtered 
    * connected component list.  This will loop through
    * the filtered CC list and assign the passed in 
    * color value to the component. 
    * Then see dlRenderColorCCs().
    * \param color color value
    */
   void dlSetFilteredCCColor(DLColor color);

   /**
    * \brief Class for iterating over all the filtered connected components.
    *
    * This is a template class, reimplemented for use in DLConnectedComponents
    * as DLConnectedComponents::const_fiterator and DLConnectedComponents::fiterator.
    *
    * Use the functions DLConnectedComponents::fbegin() and DLConnectedComponents::fend()
    * to create this type of iterator. This iterator behaves much the same as STL iterators:
    * it can be incremented, and two iterators may be compared with each other to 
    * determine if they point to the same DLComponent.
    */
   template<typename COMP, typename CVEC, typename CVECITER, typename IVEC, typename IVECITER>
      class fiterator_base : public std::iterator<std::bidirectional_iterator_tag,
      CVEC> {
         public:
      /**
       * Default constructor to allow for instantiation of
       * an const fiterator. 
       * \code

       * \endcode* DLConnectedComponents::const_fiterator fii = cc.begin();
       */
      fiterator_base() {
         ; // should add code to set iterator to known "fail" state
      }

      /**
       * Constructor used by DLConnectedComponents to create
       * the "begin()" and "end()" iterators.
       * \param vc pointer to the vector of DLComponent%s to iterate over
       * \param idxs pointer to the vector of filtered-in component indices
       * \param b true for "begin()" iterator, false for "end()" iterator
       */
      fiterator_base(CVEC * vc, IVEC * idxs, bool b)
         : myvector_p(vc), filteredIdxs_p(idxs) {

         if (b) {
            // for begin()

            // set internal DLComponent iterator to beginning of the vector of
            // DLComponents;
            filtIdx_iter = filteredIdxs_p->begin();
            myiter = myvector_p->begin();

            // then advance it to point to the first filtered component
            // (but if all the components have been filtered out, set it to the end instead)
            if ( ! idxs->empty()  ) {
               advance(myiter, *filtIdx_iter);
            } else {
               myiter = myvector_p->end();
            }

         } else {
            // for end()

            // set internal DLComponent iterator to its end (so we can check against it)
            filtIdx_iter = filteredIdxs_p->end();
            myiter = myvector_p->end();
         }

      }

      /**
       * Dereference operator: returns the individual DLComponent object that
       * the iterator is currently pointing to. Returned object
       * can not be modified.
       */
      COMP & operator* () const {
         return (*myiter);
      }

      /** 
       * Arrow operator: returns a pointer to the individual DLComponent
       * object that the iterator is currently pointing to.  Returned object
       * can not be modified.
       */
      COMP * operator-> () const{
         return &(*myiter);
      }

      /** 
       * Checks if two fiterator%s are equivalent (pointing to the same place)
       */
      bool operator!= (const fiterator_base & right) const{
         return myiter != right.myiter;
      }

      /**
       * Checks if two fiterator%s are equivalent (pointing to the same place)
       */
      bool operator== (const fiterator_base & right) {
         return myiter == right.myiter;
      }

      /**
       * Preincrement operator: advances filtered component iterator to the
       * next filtered component
       */
      const fiterator_base operator++() {
         int oldpos = *filtIdx_iter; // save current position
         ++filtIdx_iter;             // advance filtered index iterator

         if (filtIdx_iter == filteredIdxs_p->end()) {
            // if we've reached the last of the filtered components, return "end"
            myiter = myvector_p->end();
         } else {
            // advance the internal DLComponent iterator to the next filtered component
            advance(myiter, *filtIdx_iter - oldpos );
         }
         return *this;
      }

         protected:

      /// pointer to DLComponent%s
      CVEC * myvector_p; 
      /// internal iterator over DLComponent%s
      CVECITER myiter; 
      /// pointer to filtered component indices
      IVEC * filteredIdxs_p; 
      /// iterator over component indices
      IVECITER filtIdx_iter; 

   }; // class DLConnectedComponents::fiterator


   
   /// Iterator which iterates over all connected components,
   /// whether or not a filter was run
   typedef std::vector<DLComponent>::iterator iterator;

   /// \c const iterator which iterates over all connected components,
   /// weather or not a filter was run.
   /// Use this instead of the plain iterator if 
   /// the DLConnectedComponents object has itself been declared \c const.
   typedef std::vector<DLComponent>::const_iterator const_iterator;

   /// Filtered iterator which iterates over all connected components,
   /// A filtering function must first be run.
   typedef fiterator_base<const DLComponent, 
      const std::vector<DLComponent>, std::vector<DLComponent>::const_iterator,
      const std::vector<int>, std::vector<int>::const_iterator> const_fiterator;

   /// \c const filtered iterator which iterates over all connected components.
   /// A filtering function must first be run.
   typedef fiterator_base<DLComponent,
      std::vector<DLComponent>, std::vector<DLComponent>::iterator,
      std::vector<int>, std::vector<int>::iterator> fiterator;
   /**
    * Obtain an iterator to the beginning of all the connected components.
    */
   iterator begin() { return cComponents.begin(); }

   /**
    * Obtain a \c const iterator to the beginning of all the connected components.
    * Only read access is allowed with this iterator.
    */
   const_iterator begin() const {return cComponents.begin();}

   /**
    * Obtain a \c const iterator pointing just past the end of all the connected
    * components.
    */
   const_iterator end() const { return cComponents.end(); }

   /** 
    * Obtain an iterator pointing just past the end of all the connected
    * components.
    */
   iterator end() { return cComponents.end(); } 

   /**
    * Obtain an iterator over the filtered connected components;
    * this iterator is invalidated if the DLConnectedComponents filter is
    * reset, or if an additional filter operation is applied.
    */
   fiterator fbegin() { return fiterator(&cComponents,&filtCompIndices,true); }

   /**
    * Obtain an iterator to the end of the filtered connected components;
    * this iterator is invalidated if the DLConnectedComponents filter is
    * reset, or if an additional filter operation is applied.
    */
   fiterator fend() { return fiterator(&cComponents,&filtCompIndices,false); }


   /**
    * Obtain a \c const iterator over the filtered connected components;
    * this iterator is invalidated if the DLConnectedComponents filter is
    * reset, or if an additional filter operation is applied.
    */
   const_fiterator fbegin() const { return const_fiterator(&cComponents,&filtCompIndices,true); }

   /**
    * Obtain a \c const iterator to the end of the filtered connected components;
    * this iterator is invalidated if the DLConnectedComponents filter is
    * reset, or if an additional filter operation is applied.
    */
   const_fiterator fend() const { return const_fiterator(&cComponents,&filtCompIndices,false); }

   private:
   ///Flag to indicate if need to calculate convexHulls
   bool calConvexHulls;

   ///Flag to indicate if need to calculate the Moments
   bool calMoments;

   ///Vector to store the labeled connected components
   vector <DLComponent> cComponents;

   ///vector to store a mapping of all of the connected components in the origianl image
   vector<int> ccMapVector;

   ///Vector to store indices of the current Filtered connected components
   vector <int> filtCompIndices;

   ///Orginal image height 
   int imageHeight;

   ///Orginal image width
   int imageWidth;

   ///needToReCalculateCCMap is a flag to indicate 
   ///something has changed with the CC list.  If something has changed,
   ///then there is a need to recalculate the CC mappings.  
   int needToReCalculateCCMap;



};//end class

#endif //_DLCONNECTEDCOMPONENTS_H_
