#ifndef _DLDOCUMENT_H_
#define _DLDOCUMENT_H_

#include "DLPage.h"
#include "DLImage.h"
#include "DLTagList.h"
#include "DLConst.h"
#include "DLException.h"

#include <list>
#include <string>

class DLPage;

using namespace std;


/**
 * \brief Representation of a document.
 *
 * The DLDocument class is the uppermost level in the DocLib document hierarchy.
 * It is basically a container for attributes describing the document, and one, or
 * in case of multiple page documents, several pages. A DLDocument object has a
 * document ID under which the developer can store a unique index. Moreover, it
 * features a list of document tags allowing the developer to dynamically append
 * or remove additional self-defined attributes. Several constructors are available,
 * depending on the information present at the time of construction. The pages of a
 * document are stored in a STL list structure. Developers have access to this list
 * via a set of functions for appending, inserting, or deleting either one single
 * page or a list of pages.
 *
 * Transparent to the users, each DLDocument object keeps track of two kinds of DLPage
 * objects associated with it - pages on the DLDocument, and pages that point to the
 * DLDocument (which get created when copies of DLPages are made). Programmers can
 * traverse through the pages of a document using DLPagePtrIterator as below:
 *
 * \code
 * //   ... docPointer is a DLDocument* to a document already populated with pages
 *  DLDocument::DLPagePtrIterator pageIterator;
 *  for (pageIterator = docPointer->begin(); pageIterator != docPointer->end(); pageIterator++) {
 *    cout << (*pageIterator)->dlGetPageID() << endl;
 *  }
 * \endcode
 *
 * Upon its destruction, the DLDocument deletes all pages in the DLDocument. Furthermore,
 * if there exist dangling pages not contained within the DLDocument
 * that still point to the DLDocument, the DLDocument* 
 * backpointer of
 * those pages will be set to \c NULL, as the DLDocument they reference does not exist any more.
 *
 * When making a copy of DLDocument, all the pages and zones in the source document will
 * automatically be copied over to the new DLDocument object.
 * However, this is NOT simply a member-wise copy, as a new
 * hierarchy of page and zone objects is actually created recursively that duplicates
 * the page and zone hierarchy of the source DLDocument.
 *
 * It is advisable for programmers to use new operator to dynamically allocate memory
 * space when constructing DLPage%s and DLZone%s. The object destructors of DLDocument, 
 * DLPage, and DLZone will release the allocated memory space of associated objects
 * (i.e. all objects below the current object in the DLDocument hierarchy.) However, 
 * if you use the \c new operator to dynamically create an stand-alone object, you will need
 * to use the \c delete operator to reclaim the memory space it occupies.
 **/

class DLDocument
{
   friend class DLPage;

   public:

   /**
    * Iterator which iterates over all the DLPage%*'s in the document, 
    * which are stored in documentPages. 
    * Note that dereferencing this iterator gives a DLPage%*, which 
    * must be dereferenced again to give an actual DLPage object reference.
    * (See the code example in the class introduction above.)
    */
   typedef std::list <DLPage*>::iterator DLPagePtrIterator;

   /**
    * Obtain an iterator pointing to the first of all the DLPage%*'s in documentPages
    * \return an iterator pointing to the first of all the DLPage%*'s in documentPages
    */
   DLPagePtrIterator begin() { return documentPages.begin(); };

   /**
    * Obtain an iterator that addresses the location succeeding the last DLPage*s in documentPages.
    * The end() iterator is typically used in a \c for loop for bounds checking
    * (see the example in the class introduction above). Warning: dereferencing the end()
    * iterator is undefined, and will likely cause your program to crash.
    * \return an iterator that addresses the location succeeding the last DLPage*s in documentPages
    */
   DLPagePtrIterator end() { return documentPages.end(); };
	
   /**
    * Check whether the given iterator is in the valid range
    * \return true if begin() <= \a pageIter < end(); returns false otherwise
    */
   bool dlIsPageIteratorValid(DLPagePtrIterator pageIter);

   /**
    * DLDocument Constructor
    */
   DLDocument();

   /**
    * DLDocument Constructor: creates a one-page document
    * \param documentImage image for the document's page
    * \param pageID unique string ID of DLPage
    * \param docID unique string ID of DLDocument
    */
   DLDocument(const DLImage &documentImage, const string &pageID, const string &docID);

   /**
    * DLDocument Constructor: creates a one-page document from an image file
    * \param imageFileName image file name
    * \param pageID unique string ID of DLPage
    * \param documentID unique string ID of DLDocument
    */
   DLDocument(const char *imageFileName, const string &pageID, const string &documentID);

   /**
    * DLDocument Copy Constructor: all the DLPage%s/DLZone%s in the document 
    * will be copied over to the new DLDocument object.
    * The new object will  have a DLDocument hierarchy identical
    * to the source DLDocument, in which each page and zone is a new object. 
    * As DLDocument copy is NOT
    * simply a member-wise copy, it will not result in the accidental destruction 
    * of the original DLDocument's pages and zones when
    * the destructor of the DLDocument copy is invoked.
    */
   DLDocument(const DLDocument &right);

   /**
    * DLDocument Constructor
    * \param imageFileName image file name
    * \param XMLFileName XML file name
    * \param documentID unique string ID of DLDocument
    */
   // DLDocument(const char *imageFileName, const char *XMLFileName, string docID);

   /**
    * Default DLDocument Destructor
    * Upon its destruction, the DLDocument deletes all DLPages on the DLDocument. 
    * Furthermore, if there exist dangling DLPage%s that still point to the DLDocument,
    * the DLDocument%* on these DLPages will be set to \c NULL, as the DLDocument does not
    * exist any more.
    */
   virtual ~DLDocument();

   /**
    * DLDocument Assignment Operator: all the DLPage%s/DLZone%s in the document 
    * will be copied over to the new DLDocument object.
    * The new object will  have a DLDocument hierarchy identical 
    * to the source DLDocument, in which each page and zone is a new object. 
    * As DLDocument copy is NOT
    * simply a member-wise copy, it will not result in the accidental destruction 
    * of the original DLDocument's pages and zones when
    * the destructor of the DLDocument copy is invoked.
    */
   DLDocument& operator=(const DLDocument &right);

   /**
    * Get the unique ID of the document
    * \return the unique ID of the document 
    */
   string dlGetDocumentID() const { return documentID; };

   /**
    * Set the ID string of the document
    * \param docID desired unique ID of the document
    */
   void dlSetDocumentID(const string &docID) { documentID = docID; };
    
   /**
    * Get the number of pages in the document
    * \return number of pages
    */
   int dlGetNumPages() const { return static_cast<int>(documentPages.size()); };

   /**
    * Check whether the current list of pages in the document is empty
    * \return true if document contains pages
    */
   bool dlHasPages() const { return (!documentPages.empty()); };

   /**
    * Append a new page to the document at the end of the current list of pages.
    * A DLException is thrown when the DLPage to be appended is 
    * already pointing to another DLDocument as its parent.
    * The DLDocument object takes responsibility for freeing the memory occupied
    * by \a docPage. Therefore, typically \a docPage should be allocated on the heap using 
    * \c new. Taking the address of a stack object should be done with great care,
    * since the object is automatically destroyed when it goes out of scope, potentially
    * causing a dangling pointer within the document.
    * \code
    * DLDocument d;
    * {
    *   DLPage po("test1.tif", "page1");
    *   DLPage * pp = new DLPage("test2.tif, "page2"); 
    *   d.dlAppendPage(&po); // dangerous; po will soon go out of scope
    *   d.dlAppendPage(pp); // safer; pp is not destroyed unless delete called
    * }
    * \endcode
    * \param docPage new DLPage to be appended
    */
   void dlAppendPage(DLPage *docPage);

   /**
    * Append a list of new pages to the document at the end of the current list.
    * An exception occurs when any DLPage to be appended is pointing to another document.
    * The DLDocument object takes responsibility for freeing the memory occupied
    * by the DLPage%s in \a pageList.
    * \param pageList list of new DLPage%*s to be appended to the document at the end of the current list
    */

   void dlAppendPageCopy(const DLPage &docPage);
  
   /**
    * Append a list of new pages to the document at the end of the current list.
    * The DLDocument object takes responsibility for freeing the memory occupied
    * by the DLPage's in \a pageList.
    * \param pageList list of new pages to be appended to the document at the end of the current list
    */


   void dlAppendPageList(list <DLPage*> pageList);

   /**
    * Insert a new page to the document at the specified iterator position of the current list.
    * An exception occurs when the position specified is out of range or
    * when the DLPage to be inserted is currently pointing to another document.
    * The DLDocument object takes responsibility for freeing the memory occupied
    * by \a docPage.
    * \param pageIter DLPagePtrIterator pointing to the position in the current page list where the new page is to be inserted
    * \param docPage new DLPage to be inserted
    */
   void dlInsertPage(DLPagePtrIterator pageIter, DLPage *docPage);

   /**
    * Insert a new page to the document at the specified position of the current list.
    * An exception occurs when the DLPage to be inserted is pointing to another document.
    * The DLDocument object takes responsibility for freeing the memory occupied
    * by \a docPage.
    * \param cursorPosition cursor position in the current page list where the new page 
    * is to be inserted. 
    * For example, \a cursorposition == 0 inserts before the first page,
    * and \a cursorposition == 1 inserts between the first page and
    * the second page.
    * \param docPage new DLPage to be inserted
    */
   void dlInsertPage(int cursorPosition, DLPage *docPage);

   /**
    * Insert a list of new pages to the document starting from the specified iterator position in the current list.
    * An exception occurs when any DLPage to be inserted is pointing to another document.
    * The DLDocument object takes responsibility for freeing the memory occupied
    * by the DLPage%s in \a pageList.
    * \param pageIter the DLPagePtrIterator position in the current page list where the new pages are to be inserted
    * \param pageList list of new DLPage%*s to be inserted at the specified cursor position in the current list
    */
   void dlInsertPageList(DLPagePtrIterator pageIter, list <DLPage*> pageList);

   /**
    * Insert a list of new pages to the document starting from the specified position in the current list.
    * An exception occurs when any DLPage to be inserted is pointing to another document.
    * The DLDocument object takes responsibility for freeing the memory occupied
    * by the DLPage%s in \a pageList.
    * \param cursorPosition position in the current page list where the new pages are to be inserted
    * For example, \a cursorposition == 0 inserts before the first page,
    * and \a cursorposition == 1 inserts between the first page and
    * the second page.
    * \param pageList list of new DLPage*s to be inserted at the specified cursor position in the current list
    */
   void dlInsertPageList(int cursorPosition, list <DLPage*> pageList);

   /**
    * Delete a page at a given iterator position from the document.
    * This will remove the page from the list of pages on the document 
    * and the list of page backpointers of the document.
    * An exception occurs when the position specified is out of range.
    * \param pageIter DLPagePtrIterator pointing to page to be removed from the page list
    */
   void dlDeletePage(DLPagePtrIterator pageIter);

   /**
    * Delete a page at a given position from the document.
    * This will remove the page from the list of pages on the document and the list of page backpointers of the document.
    * An exception occurs when the position specified is out of range.
    * \param cursorPosition position of the page to be removed from the page list;
    * 0 is the first page, 1 is the second page, and so on.
    */
   void dlDeletePage(int cursorPosition);

   /**
    * Delete a specific page from the document.
    * This will remove the page from the list of pages on the document and the list of page backpointers of the document.
    * \param documentPage page to be deleted
    */
   void dlDeletePage(DLPage *documentPage);

   /**
    * Clear the list of pages on the document and totally remove them from the list of page backpointers of the document.
    * This also releases the memory for every child DLPage and DLZone in the hierarchy.
    */
   void dlClearPages();

   /// \name Tag List Functions
   //@{

   /**
    * Get the field value of a specified document tag
    * \param tagKey string key of the specified document tag
    * \return the field value of a specified document tag 
    */
   string dlGetTag(string tagKey) const;

   /**
    * Set a document tag. If a tag with the specified key already exists
    * in this document, it will not be overwritten unless the optional parameter
    * \a overwriteEnabled is set to \c true.  Will throw a DLException if \a overwriteEnabled is 
    * set to \c false and \a tagKey exists.
    * \param tagKey key of the document tag
    * \param tagValue value of the document tag
    * \param overwriteEnabled option for overwriting existing field (default is \c false)
    * \throw DL_Exception DL_UNKNOWN_TAG_EXCEPTION
    */
   void dlSetTag(string tagKey, string tagValue, bool overwriteEnabled = false);

   /**
    * Remove the tag with the specified key from the list of document tags.
    * \param tagKey key of the tag to remove
    */
   void dlDeleteTag(string tagKey);

   /**
    * Erase all the existing document tags
    */
   void dlClearTags() { documentTags.tagMap.clear(); };

   /**
    * Check whether a document tag with the given key exists
    * \param tagKey key of the document tag
    * \return true if <i><b>a</b> tag exists</i> with that key
    */
   bool dlIsTagSet(string tagKey) const;

   /**
    * Check whether the whole document tag list is empty
    * \return true if the document tag list is empty
    */
   bool dlIsTagListEmpty()  const { return documentTags.tagMap.empty(); };

   /**
    * Get an iterator pointing to the location of \c tagKey in the map
    * \return iterator
    */
   DLTagList::iterator dlFindTag ( string tagKey ) { return documentTags.tagMap.find(tagKey); }

   //@}

   protected:

   string documentID;

   // list of attributes
   DLTagList documentTags;

   // list of document pages
   list <DLPage*> documentPages;

   // list of pages whose DLDocument *pageDocument point to this document
   // this enables the DLDocument to keep track of its associated DLPages when its destructor is invoked
   list <DLPage*> pageBackPointers;

}; //end class

#endif //_DLDOCUMENT_H_
