#ifndef DLFILEBASEDIO_H
#define DLFILEBASEDIO_H
#include <cstdio>
#include "DLIOStream.h"


/**
 * \brief DOCLIB file-based I/O Stream 
 * 
 * DLFileBasedIO is derived from DLIOStream and it
 * implements the file based I/O system for DOCLIB.
 * This object is used by the image format classes such
 * as DLBMPImage, DLJPGImage, and so on.
 *
 * DLFileBasedIO handles the byte order (endianness) 
 * for reading/writing multi-byte-width variables in the file-based system.
 */

class DLFileBasedIO : public DLIOStream
{
   public:
   /**
    * constructor
    * \param filename file name to open
    * \param mode file mode to open as. Same as flags for fopen();
    *   \li "r": read
    *   \li "w": write 
    *   \li "rb": read binary
    *   \li "wb": write binary.
    */
   DLFileBasedIO(const char* filename, const char* mode);

   /**
    * Destructor
    */
   ~DLFileBasedIO(void);

   /**
    * dlRead reads up to a number of (size) 
    * bytes from the DOCLIB File I/O stream into the buffer. dlRead will read up
    * to the maximum buffer size or end of file, whichever comes first.
    * The buffer must be allocated prior calling the dlRead() function.
    * The number of bytes read in is returned. 
    *
    * \param buff buffer to store the data
    * \param size max buffer size to read.
    * \return size 
    */	
   int dlRead(void* buff, int size);

   /**
    * dlGets reads up to a number of (size) 
    * bytes from the DOCLIB File I/O stream into the buffer.
    * dlGets will read up
    * to the max size or until line feed or end of file.
    * The buffer must be allocated prior calling the dlRead() function.
    * The number of bytes read in is returned. 
    *
    * \param buff buffer to store the data
    * \param size max buffer size to read
    * \return size 
    */	
   int dlGets(void* buff, int size);

   /**
    * dlGetLine reads a line from the 
    * DOCLIB File I/O stream. dlGetLine will read up to an end of line or 
    * DL_PPM_MAX_LINE_LENGTH (256) or end of file. A string of
    * the read in line will be returned.
    *
    * \return string that has been read in
    */
   string dlGetLine();

   /**
    * dlWrite writes a buffer of a given
    * size to the DOCLIB File I/O stream.
    * \param buff buffer to write
    * \param size buffer size to write
    * \return size 
    */	
   int dlWrite(void* buff, int size);  

   /**
    * dlGetInt reads an int "sizeof(int) bytes"
    * from the DOCLIB File I/O stream. dlGetInt will convert the byte order 
    * of the current int value into the indicated byte order.  
    * The read in "int" value is returned. 
    *
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>
    * \return read in int value
    */	
   int dlGetInt(DLEndian endian);

   /**
    * dlGetShort reads a short "sizeof(short) bytes"
    * from the DOCLIB I/O stream. dlGetShort will convert the byte order 
    * of the current short value into the indicated byte order.  
    * The read in "short" value is returned. 
    *
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>
    * \return read in short value 
    */	
   short dlGetShort(DLEndian endian);

   /**
    * dlWriteInt writes an int to 
    * the DOCLIB I/O stream.  dlWriteInt will convert the 
    * int value into the indicated byte order and then writes 
    * it out to the DOCLIB I/O stream.
    *
    * \param intVal int value to write
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>
    */	
   void dlWriteInt(int intVal, DLEndian endian);
   /**
    * dlWriteInt writes a short to 
    * the DOCLIB I/O stream.  dlWriteInt will convert the 
    * short value into the indicated byte order and then writes 
    * it out to the DOCLIB I/O stream.
    *
    * \param shortVal short value to write
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>
    */	
   void dlWriteShort(short shortVal,  DLEndian endian);
     
   /**
    * dlClose closes the current DOCLIB File I/O stream.
    *
    * \return returns 1 if successfully closed the stream.  Otherwise, returns 0 
    */	
   int dlClose ();

   /**
    * dlFlush flushes the current DOCLIB File I/O stream. 
    *    
    * \return returns 1 if successfully closed the stream.  Otherwise, returns 0 
    */	
   int dlFlush();


   private:

   ///file descriptor
   FILE	*fp;
};

#endif // DLFILEBASEDIO_H
