#ifndef _DLGIFIMAGE_H_
#define _DLGIFIMAGE_H_

#include "DLImage.h" 
#include "DLBaseImage.h"
#include "DLBitsPerPixelConverter.h"
#include "DLSingletonDestroyer.h"
/** \cond **/
//All of the following 8 #defines should not be changed without changing the code
static const char DLGIF_GLOBAL_HEADER[] = "GIF87a";
static const int DLGIF_GLOBAL_COLOR_TABLE_SIZE = 0;        //never use global color table
static const int DLGIF_GLOBAL_COLOR_TABLE_SORT_FLAG = 0;   //never use global color table
static const int DLGIF_GLOBAL_COLOR_TABLE_FLAG = 0;        //never use global color table
static const int DLGIF_BACKGROUND_COLOR = 0;               //no background, always draw with image
static const int DLGIF_PIXEL_ASPECT_RATIO = 0;             //no aspect ratio 1x1

static const int DLGIF_EXTRA_COLOR_ENTRY_RGB = 0;
//The following #defines should not be changed without changing the code
static const int DLGIF_DATA_WIDTH = 3;
static const int DLGIF_FINISHED_PROCESSING = -8;
/**
 * \brief GIF storage class
 * 
 * DLGIFArray is only intended for use in DLGIFImage.cpp and facilitates
 * dynamic memory allocation within DLGIFImage.
 */
template<class T >
class DLGIFArray{
   private:
   /**
    * Reset data in a DLGIFArray
    */
   void defaultConstruct(){
      size=0;
      array=NULL;
   };
   public:
   /// size
   long size;
   ///T Array
   T* array;

   /**
    * Default DLGIFArray Constructor
    */
   DLGIFArray(){ defaultConstruct(); };
   /**
    * DLGIFArray Constructor
    * \param dLGIFArray_in DLGIFArray to load
    */
   DLGIFArray(const DLGIFArray<T>& dLGIFArray_in){
      defaultConstruct();
      reallocate(dLGIFArray_in.size);
      addDLGIFArray(dLGIFArray_in);
   };
   /**
    * DLGIFImage Constructor
    * \param size_in original size for the DLGIFArray
    */
   DLGIFArray(const long size_in){
      defaultConstruct();
      reallocate(size_in);
   };
   /**
    * Default DLGIFArray Destructor
    */
   virtual ~DLGIFArray(void){
      if(array!=NULL){
         delete [] array;
         array = NULL;
      }
   };
   /**
    * Used to reduce or increase the size of a DLGIFArray
    * \param size_in new size for the DLGIFArray
    */
   void reallocate(const long size_in){
      if(array!=NULL){
         delete [] array;
         array = NULL;
      }
      size=0;
      array=(T*) new T[size_in*sizeof (T)];
   };
   /**
    * Used to add an entry to the end of a DLGIFArray
    * \param entry entry to add
    */
   inline void add(const T& entry){
      array[size++]=entry;
   };
   /**
    * Used to add another DLGIFArray to the end of a DLGIFArray
    * \param dLGIFArray DLGIFArray to add
    */
   inline void addDLGIFArray(const DLGIFArray<T>& dLGIFArray){
      addDLGIFArray(dLGIFArray,0,dLGIFArray.size);
   };
   /**
    * Used to add contents of another DLGIFArray to the end of a DLGIFArray
    * \param  dLGIFArray_in DLGIFArray from which to copy contents
    * \param  position position to start in dLGIFArray_in
    * \param  size_in amount to copy from dLGIFArray_in
    */
   inline void addDLGIFArray(const DLGIFArray<T>& dLGIFArray_in,const long &position,const long &size_in){
      memcpy(&array[size],&dLGIFArray_in.array[position],size_in*sizeof (T));
      size+=size_in;
   };
   /**
    * Used to clear the contents of the current DLGIFArray
    */
   inline void reset(){ size=0; };
};//end class DLGIFArray

/** \endcond **/
/**
 * \brief GIF image I/O class.
 * 
 * Implements loading and saving for the GIF image format, and is used by DLImageFactory. 
 * See DLImage for the typical interface for loading and saving images. 
 */
class DLGIFImage : public DLBaseImage
{
   template <class SINGLETON> friend class DLSingletonDestroyer;

   private:
   /**
    * Default DLGIFImage Constructor
    */
   DLGIFImage(void);
   static DLGIFImage* gifImageReader;
   /// an instance of DLSingletonDestroyer to 
   /// use to destruct this singleton object
   static DLSingletonDestroyer<DLGIFImage> _destroyer;

   protected:
   /**
    * Default DLGIFImage Destructor
    */
   virtual ~DLGIFImage(void);

   public:

   /// \return A handle to the DLGIFImage object
   static DLGIFImage * createImageReader();

   /**
    * Load GIF Image
    * \param[in] filename name of file to be loaded
    * \param[out] image DLImage object passed by reference;
    * will be overwritten with contents of the GIF file
    */
   void dlLoadImage(const char* filename, DLImage& image);

   /**
    * Converts image_format structure to an DLImage
    * \param[in] oimage image_format
    * \param[out] dLImage reference to DLImage to copy data to
    */ 
   void dlConvertStructToDLImage(struct	DLGifImageFormat *oimage, DLImage& dLImage);


   /**
    * Load GIF Image from memory
    * \param pMemLoc pointer to beginning of memory buffer
    * \param nSize size of buffer in bytes
    * \return DLImage for the GIF Image
    */
   DLImage dlLoadImage(const void* pMemLoc, int nSize);

   /**
    * Load GIF Image
    * \param filename name of GIF image file to be opened
    * \return DLImage containing GIF image
    */
   DLImage dlLoadImage(const char* filename);

   /**
    * dlLoadImageAtPage will load a particular page from a multiple page
    * image into a DLImage
    * \param filename file name of image to be opened
    * \param pageNumber number of page in file to be opened
    * \return DLImage containing desired page
    */
   DLImage dlLoadImageAtPage(const char* filename, int pageNumber);

   /**
    * dlLoadImageAtPage will load a particular page from multiple page
    * image buffer into a DLImage
    * \param pMemLoc pointer to beginning of memory image buffer to be opened
    * \param nSize size of buffer in bytes
    * \param pageNumber number of page to be opened
    * \return DLImage containing desired page
    */
   DLImage dlLoadImageAtPage(const void* pMemLoc, int nSize, int pageNumber);

   /**
    * Save GIF Image
    * \param filename name of file to be saved
    * \param image_in DLImage to save
    */
   void dlSaveImage(const char* filename, const DLImage& image_in);

   /**
    * Save GIF Image to memory
    * \param[out] nSize size of tiff file in memory
    * \param[in] dlimage DLImage* GIF Image
    * \return pointer to the memory buffer in which the image was saved
    * \throws DL_NOT_SUPPORTED_EXCEPTION
    */
   void* dlSaveImageToMem( int& nSize, const DLImage& dlimage);

   /**
    * Determines if the image file is in the GIF format and is supported 
    * for loading by DLGIFImage. 
    * \param filename name of file to check
    * \return boolean: true if supported
    **/
   bool dlSupportImageFile(const char* filename);

   /**
    * Determines if the image in a memory buffer is in the GIF format and is supported 
    * for loading by DLGIFImage.
    * dlSupportImageFile determines if the image is supported.
    * \param pMemLoc pointer to beginning of memory buffer
    * \param size size of buffer in bytes
    * \return boolean: true if supported
    **/
   bool dlSupportImageBuffer(const void* pMemLoc, int size);

   /**
    * Get the total number of pages in the image
    * \param  pMemLoc pointer to beginning of memory buffer
    * \param nSize buffer size in bytes
    * \return number of pages
    */
   int dlGetTotalPages(void* pMemLoc, int nSize);

   /**
    * Get the total number of pages in the image
    * \param  filename file name
    * \return number of pages
    */
   int dlGetTotalPages(const char* filename);

   private:

   class DLLocalGIFImage;
   class DLGlobalGIFImage;

   /**
    * DLLocalGIFImage only meant to be used within DLGlobalGIFImage to facilitate 
    * construction of a local image wihtin a GIF, knows nothing of DLImage
    */
   class DLLocalGIFImage
      {
         /// Friend to DLSingletonDestroyer object
         /// to allow the singleton object to be 
         /// destructed at the end of the application.
         friend class DLGlobalGIFImage;
         public:
         /**
          * Default DLLocalGIFImage Constructor
          */
         DLLocalGIFImage(){};
         /**
          * Default DLGIFImage Destructor
          */
         virtual ~DLLocalGIFImage(void){};

         /**
          * Set the DLGlobalGIFImage to be used.
          * \param globalGIFImage_in to be used.
          */
         void set(const DLGlobalGIFImage* globalGIFImage_in);
         /**
          * Gets the minimum code size used for the DLLocalGIFImage
          * \return int, the minimum code size 
          */
         int getMinimumCodeSize(void) const {return minimumCodeSize;};
         /**
          * Get the data representing the DLLocalGIFImage
          * \return DLGIFArray for the DLLocalGIFImage
          */
         DLGIFArray<DL_BYTE> getImageData() const;
         /**
          * Called once for a set of DLLocalGIFImages to set static data
          */
         void initStaticData(void);

         private:
         /**
          * Used for the main functionality of creating the DLLocalGIFImage 
          */
         void makeLocalImage(void);
         /**
          * Used to facilitate construction of the local image header
          */
         void buildHeader(void);
         /**
          * Used to facilitate encoding (with no compression) of local image data
          */
         void encodeData(void);

         int leftPosition;
         int topPosition;
         int width;
         int height;

         static int minimumCodeSize;
         static int clearCode;
         static int endCode;
         static int localColorTableSize;

         static const DL_BYTE *imagedata_24;
         static long global_numpics;
         static long global_total_size;
         static int global_width;
         static int global_height;

         DLGIFArray<DL_BYTE> header;
         DLGIFArray<long> colorTable;
         DLGIFArray<DL_BYTE> byteData;
         DLGIFArray<DL_BYTE> encodedData;

         //The following #defines should not be changed without changing the code
         static const int DLGIF_LOCAL_COLOR_TABLE_SORT_FLAG = 0; //do not sort local color table
         static const int DLGIF_INTERLACE_FLAG = 0;//do not interlace
         static const int DLGIF_LOCAL_COLOR_TABLE_FLAG = 1;//always use local color table
         static const int DLGIF_ENTRY_NOT_FOUND = -8;

         static const int DLGIF_MAX_COLOR_TABLE_SIZE=256;  // >0 and <=256
         static const int DLGIF_HASH_TABLE_SIZE=DLGIF_MAX_COLOR_TABLE_SIZE+8;

      };//end class DLLocalGIFImage

   /**
    * DLGlobalGIFImage only meant to be used within DLGIFImage to facilitate 
    * construction of a a GIF from a DLImage
    */
   class DLGlobalGIFImage
      {
         public:
         /**
          * Only DLGlobalGIFImage constructor
          * \param image DLImage to be used.
          */
         DLGlobalGIFImage(const DLImage* image);
         /**
          * Default DLGIFImage Destructor
          */
         virtual ~DLGlobalGIFImage(void){};
         /**
          * Gets the DLGIFArray with all of the data for a GIF file
          * \return DLGIFArray data for DLGlobalGIFImage
          */
         DLGIFArray<DL_BYTE> getGIFImageData(void);
         /**
          * Used by DLLocalGIFImage to get the global width.
          * \return int the width
          */
         int getWidth() const {return width;};
         /**
          * Used by DLLocalGIFImage to get the global height.
          * \return int the height
          */
         int getHeight()const {return height;};
         /**
          * Used by to get the number of pixels in the global image.
          * \return long the number of pixels
          */
         long getNumpics()const {return numpics;};
         /**
          * Used by to get the total size (in bytes) in the global image.
          * \return long the total size
          */
         long getTotalSize()const {return totalSize;};
         /**
          * Used to get the image data in the DLImage
          * \return DL_BYTE* the image data int the global DLImage
          */
         const DL_BYTE* getDLImageData() const {return globalDLImage->dlGetImageData();};

         private:
         /**
          * Used to facilitate construction of the GIF's screen descriptor
          */
         void buildScreenDescriptor(void);
         const DLImage* globalDLImage;
         DLGIFArray<DL_BYTE> screenDescriptor;
         DLLocalGIFImage dLLocalGIFImage;

         bool quantized;
         int width;
         int height;
         long numpics;
         long totalSize;
         long currentPosition;

         int bitsPerPixel;  //minimum code size always equals this

         //All of the following 8 #defines should not be changed without changing the code
         static const int DLGIF_IMAGE_BLOCK_CODE =0x2C;
         static const int DLGIF_TERMINATOR_BLOCK_CODE = 0x3B;

      };//end class DLGlobalGIFImage
};//end class DLGIFImage

static DLGIFImage * dlGifProcessor = DLGIFImage::createImageReader();

#endif //_DLGIFIMAGE_H_
