/**
 * Prior to going through the code related to GIF load and save, 
 * please read about the GIF file format.  A well structured 
 * presentation appears in:
 *      "Compressed Image File Formats" by John Miano
 *      ISBN: 0-201-60443-4
 */
#include "DLLogger.h"
#include "DLException.h"

/** \cond **/

///LZW Table structure
struct DL_LZW_table {
   ///Count
   int         count;
   ///Out
   unsigned char  *out;
};

/**
 * Gif Image Format
 */
struct DLGifImageFormat {
   ///pointer to pixel data, can be of type int, float, unsigned char etc. 
   void *image;
   /** determines the type of data pointed to when casting "image" pointer above    
       0 -> unsigned char * 
       1 -> char *
       2 -> unsigned int *
       3 -> int *  
       4 -> float *  
       10 -> FILE *, data is unsigned char
       11 -> FILE *, data is char
       12 -> FILE *, data is unsigned int
       13 -> FILE *, data is int 
       14 -> FILE *, data is float */
   int type;
   /** Endian byte order
       0 -> little endian; first characters in tiff image are "II"
       1 -> big endian; first characters in tiff image are "MM" */
   int byte_order;

   /** Type of compression to be used.  Defined by TIFF documentation.
       1 -> uncompressed (dump mode)
       2 -> CCITT 1D (modified Huffman RLE)
       3 -> CCITT Group 3 
       4 -> CCITT Group 4 
       5 -> LZW
       32773 -> PackBits */
   int compression;

   /// The pixel width of the image; iCols in TwoBit
   int width;

   /// The pixel height of the image; iRows in TwoBit 
   int height;

   /** The number of color (image) channels; the dimensionality of the color space
       1 -> grayscale and binary
       3 -> color (RGB or YUV) */
   int numchan;
   /** The number of image frames; the number of images at the same spatial
       resolution but representing successive times.  Used for video or animated
       gifs and other multi-image files.  Single image files have numframes = 1. */

   int numframes;
   /** The number of bits of precision; the number of bits representing each
       channel of a single pixel
       1 -> binary
       8 -> grayscale or color */
   int bits_per_chan;
   /** This value only applies to binary images.
       0 -> one byte per pixel
       1 -> one bit per pixel; 8 pixels per byte */
   int packed;
   /** TIFF definition of photometric interpretation (TIFF tag)
       0 -> white is zero (only for binary or grayscale images)
       1 -> black is zero (only for binary or grayscale images)
       2 -> RGB
       3 -> Pallet RGB
       4 -> mask
       5 -> seperated
       6 -> YCbCr (YUV) */
   int photoInterpretation;
   /** The units of resolution
       2 -> inches
       3 -> centimeters */
   int unit_resolution;
   ///The horizontal resolution of the image; "fHorizontalRes" in TwoBit
   int xresolution;
   /// The vertical resolution of the image; "fHorizontalRes" in TwoBit
   int yresolution;

   /** A pointer to m (m = numchan) integer values that determine the ratio
       of spatial resolution for different image channels. For example, in the
       YCbCr color space used by MPEG video the ratio is 4:1:1, represented by
       the values 4, 1, 1.  If a single channel is used in the image or all channels
       have the same scale, a NULL pointer is used.  If color channels are subsampled,
       this will be indicated here.  It is seldom used and not supported by most code.
       It is always set to NULL. */
   int *chan_ratio;
   ///A character pointer for a descriptive string about the image (s).  Never used. 
   char *desc;
   /// We have no idea what this is.  Never used.
   char filetype[5];
   ///A pointer for a user definition.  Never used.
   void *user_def0;
   /// A value for a user definition.  Never used. 
   int user_def1;
   /** Think of palette as a mapping from the image to a color chart.  In actuality,
       it is a pointer to another image structure.  Hence, the "decoded image" has a
       pointer to the "palette image".  The palette image structure is an image of
       dimensions NUMCOLORS x 1 where the number of unique colors in the decoded image
       determines the width of the palette image.  Each color has an RGB value, so the
       "numchan" value in the palette image is 3.  The color of the pixel at location
       (n, 0) in the palette image is the color to replace any pixel with the value n
       from the decoded image where n is between 0 and NUMCOLORS-1, inclusive.  In order
       to use the palette, the decoded image should be of type 0 or 2 (type 3 might work).
       Each pixel value is represented by an index that points to the palette image.  This
       value is rarely used. */
   struct DLGifImageFormat *palette;
};



/// \brief GIF Utility Class
///
/// For internal use by DLGIFImage. See DLImage for interface for loading and saving images.
class DLGIFUtilities
{
   public:

   /**
    * Paste Image transparent
    * \param image_overlay
    * \param image_out
    * \param origin
    * \param transparent
    * \return int
    */
   static int image_paste_transparent (struct DLGifImageFormat *image_overlay, 
                                       struct DLGifImageFormat **image_out, 
                                       int *origin, unsigned char transparent);
   /**
    * read gif image
    * \param buffer
    * \param buffSize
    * \param image_in
    * \return int
    */
   static int image_read_gif(const unsigned char *buffer, int buffSize, struct DLGifImageFormat **image_in);
   /**
    * paste image
    * \param image_overlay
    * \param image_out
    * \param origin
    * \return int
    */
   static int image_paste (struct DLGifImageFormat *image_overlay, struct DLGifImageFormat **image_out, int *origin);
   /**
    * Realign Gif image
    * \param fimage
    * \param error_locs
    * \param num_fragments
    * \return int
    */
   static int image_GIF_realign (struct DLGifImageFormat *fimage, int *error_locs, int num_fragments);
   /**
    * Deintelace gif image
    * \param fimage
    * \return int
    */
   static int image_GIF_deinterlace (struct DLGifImageFormat *fimage);
   /**
    * Find the next clear code 
    * \param inbuf
    * \param numbytes
    * \param codesize
    * \param bytepos
    * \param bitpos
    * \return int
    */
   static int buffer_find_next_clear_code_new (unsigned char *inbuf, int numbytes,int codesize, int *bytepos, int *bitpos);
   /**
    * buffer resync gif block 
    * \param buf
    * \param offset
    * \param bufsize
    * \param key
    * \return int
    */
   static int buffer_resync_gif_block (unsigned char *buf, int offset, int bufsize, int key);
   /**
    * Buffer find far processed
    * \param fbuf
    * \param findx
    * \param fmax
    * \param eindx
    * \return int
    */
   static int buffer_find_far_processed (unsigned char *fbuf, int findx, int fmax, int eindx);
   /**
    * buffer LZW decode 
    * \param inbuf
    * \param byte_offset
    * \param bit_offset
    * \param numbytes
    * \param start_codesize
    * \param fimage
    * \param pixloc
    * \param eoi_flag
    * \param retVal
    * \return int
    */
   static int buffer_LZW_decode_new (unsigned char *inbuf, int byte_offset, 
                                     int bit_offset,int numbytes, 
                                     int start_codesize, struct DLGifImageFormat *fimage, 
                                     int *pixloc, int *eoi_flag, int *retVal);
   /**
    * buffer parse gif block
    * \param buf
    * \param bufindx
    * \param bufsize
    * \param obuf
    * \param numread
    * \param key
    * \return int
    */
   static int buffer_parse_gif_block_new (unsigned char *buf, int bufindx, 
                                          int bufsize, unsigned char *obuf, 
                                          int *numread, int *key);
   /**
    * palette adjust transparent
    * \param palette
    * \param transparent_value
    * \return int
    */
   static unsigned char palette_adjust_transparent (struct DLGifImageFormat *palette, 
                                                    unsigned char transparent_value);
   /**
    * Initialize LZW table
    * \param table
    * \param size
    * \param tbuf
    * \param tindx
    * \return int
    */
   static int initialize_LZW_table(struct DL_LZW_table *table, int size, unsigned char *tbuf, int *tindx);
   /**
    * get next code
    * \param buffer
    * \param indx
    * \param codesize
    * \param inbuf
    * \param bufindx
    * \return int
    */
   static int getnextcode(int *buffer, int *indx, int codesize, unsigned char *inbuf, int *bufindx);
   /**
    * Create Image
    * \param image_out
    * \param size
    * \param numchan
    * \param type
    * \return int
    */
   static int image_create (struct DLGifImageFormat **image_out, int *size, int numchan, int type);
   /**
    * Copy Image
    * \param image_in
    * \param image_out
    * \return int
    */
   static int image_copy (struct DLGifImageFormat *image_in, struct DLGifImageFormat **image_out);
};

/** \endcond **/

