#ifndef DLIMAGEFACTORY_H
#define DLIMAGEFACTORY_H

#include "DLImage.h"
#include "DLBaseImage.h"
#include "DLException.h"
#include "DLSingletonDestroyer.h"
#include <vector>
#include <map>
#include <string>
using namespace std;


/** \cond **/

struct dlLtstr
{
   bool operator()(const string& s1, const string& s2) const
   {
      return s1 <  s2;
   }
};

/** \endcond **/

/**
 * \brief A singleton object which handles loading 
 * and saving of images for DLImage.
 *
 * DLImageFactory is used internally by DOCLIB to support the loading and saving
 * of images in a variety of file formats. DOCLIB programmers should first see DLImage
 * for the typical DOCLIB
 * programming interface for loading and saving images.
 *
 * DLImageFactory keeps tracks/manages all supported image objects in 
 * in a list and is populated during run time. 
 * DLImageFactory provides a registration API which allows image objects, in the
 * form of DLBaseImage base class pointers 
 * to register and be included as a supported DOCLIB image type.
 *
 * Each supported image format is subclassed from DLBaseImage (e.g. DLBMPImage, DLTIFFImage) 
 * and implemented as a singleton: the single static instance of the image format object
 * must be initialized at file scope in the image format class. To ensure that all
 * supported image formats are available to the DOCLIB programmer, DLImage includes 
 * the header file DLSupportedImages.h, which in turn includes the several image format
 * class header files. Including the class header file triggers the construction of the 
 * image format object; upon construction, the object registers itself with DLImageFactory
 * by providing a base class DLBaseImage pointer.
 * 
 * When one of the DLImage load functions is called, the request is passed to DLImageFactory,
 * which uses dlIdentifyImageType() to determine if the image file or buffer is supported
 * by any of the registered image formats. 
 * If the load function is called for a file name, the DLImageFactory first attemps 
 * to load it by its extension. If that fails, or if the load function is called for 
 * a memory buffer, the DLImageFactory will loop through all of the registered image format
 * pointers, and ask each image format object if it can load the file or buffer in question.
 * The first one to match will load the image; if none can load it, a DLException will 
 * be thrown.
 */

class DLImageFactory
{
   /// Friend to DLSingletonDestroyer object
   /// to allow the singleton object to be 
   /// destructed at the end of the application.
   template <class SINGLETON> friend class DLSingletonDestroyer;
   private:

   /// This is the ONLY instance of DLImageFactory
   static DLImageFactory *iFactory;
   /// an instance of DLSingletonDestroyer to 
   /// use to destruct this singleton object
   static DLSingletonDestroyer<DLImageFactory> _destroyer;

   /// Vector of supported/registered images
   vector <DLBaseImage*> imageFormats;

   /// A map of file extensions to an actual supported image object.
   map < string, DLBaseImage*, dlLtstr > imageExtensionMap;
   private:

   /**
    * Constructor is private because this is a singleton object
    * and the only way to create this object is to call the 
    * static dlGetInstance() function.
    */
   DLImageFactory(void){};

   /**
    * Destructor is private because this object should not be destructed.
    */
   virtual ~DLImageFactory(void); 

   public:

   /**
    * Loads a particular page of a multi-page image from a file.
    * DLImage factory will call the individual image
    * types to perform this task.
    * \param filename name of image file to be opened
    * \param pageNumber the page number to be opened
    * \return DLImage of particular page
    */
   DLImage dlLoadImageAtPage(const char* filename, int pageNumber) ;

   /**
    * Loads an image from a memory buffer at a particular page.
    * DLImage factory will call the individual image
    * types to perform this task.
    *
    * \param pMemLoc pointer to beginning of memory image buffer to be opened
    * \param nSize size of memory buffer in bytes
    * \param pageNumber number of page to be opened
    * \return DLImage of particular page
    */
   DLImage dlLoadImageAtPage(const void* pMemLoc, int nSize, int pageNumber) ;

   /**
    * Loads an image from a file.
    * The function first checks if the image object 
    * specified (imageType) is the correct image object.  
    * If an image object is not specified or the specified
    * image object is incorrect, DLImageFactory then 
    * loops through all the registered image objects 
    * and determines whether the file is of that type
    *  by calling DLBaseImage::dlSupportImageFile(). 
    * Once the supported image object is found, it will
    * perform the image loading.
    *
    * \param filename file name to open
    * \param imageType Image format object used to load the file
    * \return DLImage from file
    */

   DLImage dlLoadImage(const char* filename, DLBaseImage* imageType = NULL);

   /**
    * Loads an image from a memory buffer.
    * The function first checks if the image object 
    * specified (imageType) is the correct image object.  
    * If an image object is not specified or the specified
    * image object is incorrect, DLImageFactory then 
    * loops through all the registered image objects 
    * and determines whether the file is of that type
    *  by calling DLBaseImage::dlSupportImageFile(). 
    * Once the supported image object is found, it will
    * perform the image loading.
    * \param pMemLoc pointer to beginning of buffer
    * \param nSize size of buffer in bytes
    * \param imageType Image format object used to load the file
    * \return DLImage from memory buffer
    */	
   DLImage dlLoadImage(const void* pMemLoc,const int nSize, DLBaseImage* imageType);

   /**
    * Saves an image to a file.
    * If an image format object
    * is specified, it will be used to do the saving.  
    * Otherwise, the image format object correspoding to the 
    * extension of the filename provided is used to perform the save.
    * \param image DLImage to save to file
    * \param filename file name to save to
    * \param imageType Image format object used to save the file
    */	

   void dlSaveImage(const DLImage& image, const char* filename, DLBaseImage* imageType = NULL);

   /**
    * Save an image to memory.
    *
    * The image format object must be sent (unlike when writing to file)
    * \param[in] image DLImage to save to memory buffer
    * \param[out] nSize this is where the buffer length will be returned)
    * \param[in] imageType pointer to image format object
    * \return pointer to memory buffer in which the image was saved
    * \throw DL_NOT_SUPPORTED_EXCEPTION
    */
   void* dlSaveImageToMem(const DLImage& image, int& nSize, DLBaseImage* imageType);

   /** 
    * Dump information about all supported image formats to the DLLogger at level
    * DLLogger::DL_INFO.
    */
   void dlShowAllRegistered();

   /**
    * Return all supported image objects.  User can loop through the
    * vector to access each of the objects.
    * \return vector of all supported image objects as base class (DLBaseImage) pointers
    */
   vector <DLBaseImage*> dlGetRegisteredImageObjects() const {return this->imageFormats;};

   /**
    * Registers a supported image type
    * with the image factory.  DLImageFactory will use 
    * all the registered image objects when loading and saving images.
    * This function will create the single instance of the DLImageFactory object
    * if it does not yet exist.
    * \param image pointer to image format object to be registered.
    */
   static void dlRegisterImageType(DLBaseImage* image);

   /**
    * Get the total number of pages in an image stored in a memory buffer.
    * \param pMemLoc pointer to beginning of buffer
    * \param nSize size buffer in bytes
    * \return the total number of pages
    */
   int dlGetTotalPages(void* pMemLoc, int nSize);

   /**
    * Get the number of pages in an image file.
    * \param filename name of image file to check
    * \return the total number of pages 
    */
   int dlGetTotalPages(const char* filename);

   /**
    * Identifies the type of image stored in a memory buffer.
    * Returns a base class pointer to the appropriate image format
    * object, or a NULL pointer if no image format matches.
    * \param pMemLoc pointer to beginning of memory buffer
    * \param nSize size of buffer in bytes
    * \return base class (DLBaseImage) pointer to image format object
    */
   DLBaseImage* dlIdentifyImageType(const void* pMemLoc,const int nSize) ;
  
   /**
    * Identifies the type of image stored in a file.
    * Returns a base class pointer to the appropriate image format 
    * object, or a NULL pointer if no image format matches.
    * \param filename name of file to check
    * \return base class (DLBaseImage) pointer to image format object
    */
   DLBaseImage* dlIdentifyImageType(const char* filename) ;

   /** 
    * Returns a pointer to the only instance of DLImageFactory.
    * If the DLImageFactory instance does not exist, it will create an instance and return it.
    **/
   static DLImageFactory* dlGetInstance();

};


#endif // DLImageFactory_H
