#ifndef _DLIMAGEHEAD_H_
#define _DLIMAGEHEAD_H_

#include <ostream>

#include "DLTagList.h"
#include "DLConst.h"
#include "DLColor.h"

/**
 * \brief Image header object for DLImage
 *
 * The DLImageHead class holds all of the generic image header information needed to 
 * perform image processing functions
 */
class DLImageHead
{
   public:
   /**
    *  DLImageHead Default Constructor
    */
   DLImageHead();

   /**
    *  DLImageHead Copy  Constructor
    */
   DLImageHead(const DLImageHead & orig);

   /**
    *  A selection of possible image bit depths
    */
	enum DLBitDepths
	{
		DL_BITDEPTH_UNKNOWN = -1,
		DL_BITDEPTH_1  = 1,
		DL_BITDEPTH_8  = 8,
		DL_BITDEPTH_24 = 24
	};

   /**
    *  A selection of possible data alignments
    */
	enum DLDataAlignments 
	{
		DL_UNKNOWN_ALIGN = -1,
		DL_1BYTE_ALIGN = 1,
		DL_4BYTE_ALIGN = 4
	};

   enum DLResolutionUnits {
	   DL_RESOLUTION_UNITLESS = 0,
	   DL_RESOLUTION_DPI = 1
   };


   /**
    *  Assignment operator
    * \param right DLImageHead 
    */
   const DLImageHead & operator=(const DLImageHead & right);

   /**
    *  DLImageHead destructor
    */
   virtual ~DLImageHead();

   /// \brief image tags
   ///
   /// A set of user-defined tags containing image metadata.
   DLTagList imageTags;

   ///image width
   int width;

   ///image height
   int height;

   ///\brief image bit depth
   ///
   /// Number of bits per image pixel; can be one of:
   /// <UL>
   /// <LI> DLImageHead::DL_BITDEPTH_1
   /// <LI> DLImageHead::DL_BITDEPTH_8
   /// <LI> DLImageHead::DL_BITDEPTH_24
   /// </ul>
   DLBitDepths depth;

   /// \brief image channels
   ///
   /// Number of data channels in the image; usually 3 for 24-bit color
   /// RGB images, and 1 for binary, grayscale or palettized images.
   int channels;

   /// \brief image color model
   ///
   /// Can be one of:
   /// <UL>
   /// <LI> DLColor::DL_BINARY
   /// <LI> DLColor::DL_GRAY
   /// <LI> DLColor::DL_RGBCOLOR
   /// <LI> DLColor::DL_YUVCOLOR
   /// </ul>
   DLColor::DLColorTypes colorModel;

   /// \brief image row size
   ///
   /// Number of bytes needed to store the raw data of a single row of the image.
   int rowSize;

   /// \brief image data alignment
   /// 
   /// Can be either:
   /// <UL>
   /// <LI>DLImageHead::DL_1BYTE_ALIGN
   /// <LI>DLImageHead::DL_4BYTE_ALIGN.
   /// </UL>
   DLDataAlignments dataAlignment;

   /// Not currently used.
   int dataOrder;

   /// \brief horizontal resolution
   ///
   /// Measurement units are indicated by DLImageHead::resolutionFlag.
   float horizontalResolution;

   /// \brief vertical resolution
   ///
   /// Measurement units are indicated by DLImageHead::resolutionFlag.
   float verticalResolution;

   /// \brief image resolution flag
   ///
   /// Indicates the units of measurement for the image resolution values.
   /// Upon opening an image of any format containing resolution information,
   /// DOCLIB automatically converts the resolution to dots-per-inch
   /// (or unitless, if the image does not indicate the units used).
   /// \li \c true if the image resolution is in dpi
   /// \li  \c false if the resolution is unitless
   /// Use DLImageHead::resolutionUnits instead
   /// <br>This is in <b>Phase One</b> of the <i>Documented Deprecation Process</i>
   bool resolutionFlag;

   /// \brief image resolution units
   ///
   /// Indicates the units of measurement for the image resolution values.
   /// Upon opening an image of any format containing resolution information,
   /// DOCLIB automatically converts the resolution to dots-per-inch
   /// (or unitless, if the image does not indicate the units used).
   /// \li DLImageHead::DL_RESOLUTION_DPI
   /// \li DLImageHead::DL_RESOLUTION_UNITLESS
   DLImageHead::DLResolutionUnits resolutionUnits;

   /// \return Number of bytes used to store the raw data for each row of this image.
   int dlGetRowSize() const {

      int lrowSize = 0;
      if (depth == DL_BITDEPTH_8)
         lrowSize = width;
      else if (depth == DL_BITDEPTH_24)
         lrowSize = width * 3;
      else if (depth == DL_BITDEPTH_1)
         lrowSize = (width + 7)/8;

      return lrowSize;
   };

};//end class

/// Stream output of a DLImageHead.
/// Allows for the following:
/// \code
/// DLImage i("test.tif");
/// cout << i.dlGetImageHead();
/// \endcode
/// \relates DLImageHead
/// \param strm the output strm; goes on the left-hand side of the operator
/// \param head the image header to output
/// \return a reference to the same stream; allows for chaining of multiple
///         objects in the stream output statement as in the above example.
std::ostream & operator<< (std::ostream& strm, const DLImageHead & head);

#endif //_DLIMAGEHEAD_H_
