#ifndef _DLIMAGESTATS_H_
#define _DLIMAGESTATS_H_


#include <iostream>

/**
 * \brief Image statistics object.
 * 
 * The DLImageStats object contains statistics about an image, such as
 * the mean and standard deviation of pixel values. It is generally
 * created by calling DLImage::dlGetStats().
 * 
 * All statistics are calculated for the three color channels: 
 * red, green and blue, respectively. If an image is grayscale or
 * binary, the three entries for each statistic will be set equal to 
 * each other.
 * All statistics are public data members, and can be accessed 
 * as illustrated in the following example:
 * \code
 * DLImage i("test.tif");
 * DLImageStats st = i.dlGetStats();
 * cout << "Mean of red channel: << st.mean[0]
 *      << "; mean of green channel: " << st.mean[1]
 *      << "; mean of blue channel: " << st.mean[2]
 *      << endl;
 * \endcode
 */
class DLImageStats {
   public:
  
   /**
    * Mean pixel values for each color channel. For grayscale and
    * binary images, each entry will be equal.
    * \f[ \bar{X} = \frac{1}{N} \sum_{i=1}^N x_i \f]
    * where <i>N</i> is the number of pixels and \f$x_i\f$ are the
    * individual pixel values.
    */
   double mean[3]; 
  
   /**
    * Median pixel values per channel.
    * \f[ \tilde{X} = \left\{ \begin{array}{ll} Y_{(N+1)/2} & N \mbox{odd} \\
    * 				\frac{1}{2} \left( Y_{N/2} + Y_{(N/2)+1} \right) & N \mbox{even}
    *			\end{array} \right. \f]
    * where \f$Y_k\f$ is the <i>k</i>th order statistic.
    */
   double median[3];

   /**
    * Standard deviations of pixel values.
    * \f[ S = \sqrt{ \frac{1}{N-1} \sum_{i=1}^N (x_i - \bar{X})^2 } \f]
    */
   double sdev[3];

   /**
    * Coefficient of skewness of pixel values. This is the third
    * central moment divided by the cube of the sample standard
    * deviation.
    * \f[ \alpha_3 = \frac{ \sum_{i=1}^N (x_i - \bar{X})^3 }{ S^3 } \f]
    */
   double skewness[3];

   /**
    * Coefficient of kurtosis of pixel values. This is the fourth
    * central moment divided by the fourth power of the sample standard
    * deviation.
    * \f[ \alpha_4 =  \frac{ \sum_{i=1}^N (x_i - \bar{X})^4 }{ S^4 } \f]
    */
   double kurtosis[3];

};

/// Makes a nicely formatted ouput of a DLImageStats object.
std::ostream &  operator<< (std::ostream& , const DLImageStats & );

#endif
