#ifndef _DLJPGIMAGE_H_
#define _DLJPGIMAGE_H_

// Default JPEG compression quality
const static short int DL_JPEG_CMPRESSION_QUALITY = 95;

//#include "DLTypes.h"
#include "DLImage.h"
#include "DLBaseImage.h"
#include "DLSingletonDestroyer.h"
//#include "DLLogger.h"

/// \brief Stores JPEG color palette information
typedef struct {
   /// Color 3-or 4 tuple
   unsigned char color[4];
   ///Number of occurences in some image
   unsigned int count;
}DLColorCounts;

#if defined (WIN32) || defined(_WINDOWS) || defined(__WIN32__) || defined(_Windows)
///An external JPEG decompression structure
/*extern*/
struct jpeg_decompress_struct;
struct jpeg_compress_struct;

#else
extern "C" {
#include "jpeglib.h"
}
#endif


/**
 * \brief JPEG image I/O class.
 *
 * Implements loading and saving for the JPEG image format, and is used by DLImageFactory.
 * See DLImage for the typical interface for loading and saving images.
 */
class DLJPGImage : public DLBaseImage
{
   /// Friend to DLSingletonDestroyer object
   /// to allow the singleton object to be 
   /// destructed at the end of the application.
   template <class SINGLETON> friend class DLSingletonDestroyer;
   private:
   /**
    * Default DLJPGImage Constructor
    */
   DLJPGImage();
   ///Only instance of DLJPGImage
   static DLJPGImage* jpgImageReader;
   /// an instance of DLSingletonDestroyer to 
   /// use to destruct this singleton object
   static DLSingletonDestroyer<DLJPGImage> _destroyer;

   //Set to true to decompress a JPEG using decompression
   bool quantized;

   //next value used by jpeg_start_decompress for quantization
   int desired_number_of_colors;

   //next value set by jpeg_start_decompress() provided 
   //quantization is requested
   int actual_number_of_colors;

   //Pointer to an array of color values and counts defined in dlLoadImage()
   DLColorCounts *palette;

   //next value set by jpeg_read_header
   int color_space;

    
   /**
    * dlSetupMemSrcMgr is a private function to setup the
    * memory based decompresion manager
    * \param cinfo jpeg_decompress_structure to be filled in.  
    * \param pMemLoc memory Image buffer to be opened

   */
   void dlSetupMemSrcMgr(jpeg_decompress_struct* cinfo, const void* pMemLoc, int nSize);

   /**
    * dlSetupMemDstMgr is a private function to setup the
    * memory based compresion manager
    * \param cinfo jpeg_compress_structure to be filled in.  
    * \param pMemLoc memory Image buffer to be written to

   */
   static void dlSetupMemDstMgr(jpeg_compress_struct * cinfo, const void* pMemLoc);
    
   DLImage dlLoadImage(jpeg_decompress_struct& cinfo);

   protected:
   /**
    * Default DLJPGImage Destructor
    */
   virtual ~DLJPGImage();

   public:

   /**
    * \return A handle to the DLJPGImage object
    */
   static DLJPGImage * createImageReader();

   /**
    * Load JPG Image
    * \param value quantize flag
    * \return void
    */
   //void setQuantized(bool value) { quantized = value; }

   /**
    * Load JPG Image
    * \param value desired number of colors
    * \return void
    */
   //void setDesiredColors(int value) { desired_number_of_colors = value; }

   /**
    * \return actual number of colors
    */
   int dlGetActualColors() const { return actual_number_of_colors; }

   DLColorCounts* dlGetPalette() const { return palette;}

   /**
    * \return color space
    */
   int dlGetColorSpace() const { return color_space; }

   void dlAllocatePalette(int numberOfColors);

   void dlFreePalette();

   /**
    * Load JPG image
    * \param[in] filename name of file to load
    * \param[out] image DLImage object passed by reference;
    * will be overwritten to contain image from JPG file.
    */
   void dlLoadImage(const char* filename, DLImage& image);

   /**
    * Load JPG image from memory
    * \param  pMemLoc pointer to beginning of memory buffer
    * \param nSize size of the memory buffer in bytes
    * \return DLImage of the JPG image
    */
   DLImage dlLoadImage(const void* pMemLoc, int nSize);

   /**
    * Load JPG Image
    * \param filename name of JPG image file to be opened
    * \return DLImage of the JPG Image
    */
   DLImage dlLoadImage(const char* filename);

   /**
    * Loads a particular page from a multiple page
    * image.
    * \param filename name of image file to be opened
    * \param pageNumber number of page to be opened
    * \return DLImage of a page in a JPG file
    */
   DLImage dlLoadImageAtPage(const char* filename, int pageNumber);

   /**
    * Loads a particular page from a multiple page
    * image in a memory buffer into a DLImage
    * \param pMemLoc pointer to beginning of memory image buffer to be opened
    * \param nSize size of the memory buffer in bytes
    * \param pageNumber number of page to be opened
    * \return DLImage of a page in a JPG buffer
    */
   DLImage dlLoadImageAtPage(const void* pMemLoc, int nSize, int pageNumber);

   /**
    * Save JPG Image to memory. See documentation for DLImage code example
    * \param[out] nSize size of tiff file in memory
    * \param[in] image DLImage* JPG Image
    * \return pointer to the memory buffer in which the image was saved
    */
   void *dlSaveImageToMem(int &nSize, const DLImage& image);

   /**
    * Save JPG Image to memory buffer with specific quality factor.
    * The quality parameter must be in the range [0,100] where
    * 0 gives the lowest image quality and highest compression, and
    * 100 gives the best image quality but least effective compression.  
    * See below code example for use:
    * \code
    * DLImage img ("a.jpg");
    * int size;
    * void *buff = (DLJPGImage::createImageReader())->dlSaveImageToMem(size, img, 90);
    * \endcode
    * \c size will now contain the size of the memory block stored in \c buff
    * \param[out] nSize size of jpg file in memory
    * \param[in] image DLImage* JPG Image
    * \param[in] imageQuality int qualitiy value [0,100] (see above for explanation)
    * \return pointer to the memory buffer in which the image was saved
    */
   static void *dlSaveImageToMem(int &nSize, const DLImage& image, const int imageQuality );

   /**
    * Save JPG Image. 
    * Uses default quality factor of 95.
    * \param filename name of file to be saved
    * \param image image to be saved
    */
   void dlSaveImage(const char* filename, const DLImage& image);

   /**
    * Save JPG Image with specific quality factor.
    * The quality parameter must be in the range [0,100] where
    * 0 gives the lowest image quality and highest compression, and
    * 100 gives the best image quality but least effective compression.
    * \param filename name of file to be saved
    * \param image image to be saved
    * \param imageQuality int image compression quality
    */
   static void dlSaveImage(const char* filename, const DLImage& image, const int imageQuality );

   /**
    * Determines if the image file is in the JPG format and 
    * is supported for loading by DLJPGImage.
    * \param filename  name of file to check
    * \return boolean: true if supported
    **/
   bool dlSupportImageFile(const char* filename);

   /**
    * Determines if the memory buffer is in the JPEG format and
    * is supported for loading by DLJPGImage. 
    * \return boolean: true if supported
    **/
   bool dlSupportImageBuffer(const void* pMemLoc, int size);

   /**
    * Get the total number of pages in the image
    * \param  pMemLoc pointer to beginning of memory buffer
    * \param nSize buffer size in bytes
    * \return number of pages
    */
   int dlGetTotalPages(void* pMemLoc, int nSize);

   /**
    * Get the total number of pages in the image
    * \param  filename file name
    * \return number of pages
    */
   int dlGetTotalPages(const char* filename);

};//end class
static DLJPGImage * dlJpgProcessor = DLJPGImage::createImageReader();
#endif //_DLJPGIMAGE_H_
