
#ifndef DLLOGGER_H
#define DLLOGGER_H

#include "DLSingletonDestroyer.h"

#include <string>
#include <cstdlib>
#include <fstream>
using namespace std;

/**
 * This is all the Log Severity levels
 * doclib currently supports.  
 */
enum DLSeverity  {
   DL_DEBUG = 1,
   DL_INFO = 2,
   DL_CRITICAL = 3,
   DL_LOG_OFF = 4
};

///Log output Type 
enum DLLogType  {
   DL_STDERR = 1,
   DL_STDOUT = 2,
   DL_FILEOUT = 3
};



/**
 * \brief Debug, informational, and error message logging.
 * 
 * Please DO NOT use \c printf or \c cout to do system outputs!!
 * 
 * The purpose of having this logger is to centralize the 
 * logging of debug, informational, and error
 * messages. Each log statement will have a severity associated with it.
 * Users may filter logger output based on the severith level.
 * Logging filters can be set using a the 
 * DLLogger::dlSetLogSeverity() function during 
 * the execution of the application.  By default, the severity is set 
 * to DL_CRITICAL to allow only critical errors to be displayed.  The logger
 * can also be turned off completely by setting the severity to DL_LOG_OFF.  
 *  
 * The user can also turn off the logger feature completely 
 * by the defining the compile option "DL_LOGGER_OFF" and recompiling DOCLIB.
 * If performance is an issue to your application, turn off the logger.
 * 
 * Here are the different type of filters/severities DLLogger supports: 
 * <ul> 
 * <li> DL_DEBUG : Will print out all log messages
 * <li> DL_INFO : Will print out INFO, CRITICAL
 * <li> DL_CRITICAL : Will only print CRITICAL 
 * <li> DL_LOG_OFF : Supress all log output
 * </ul>
 * 
 * This is how to use the DLLogger.
 * 
 * 1. Add this include: 
 * \code
 * #include "DLLogger.h"
 * \endcode
 * 
 * 2. Define the current system log severity
 * \code
 * dlLogger.dlSetLogSeverity(DL_INFO);
 * \endcode
 * 
 * Note that \c dlLogger is a globally-available static instance of DLLogger, 
 * and is automatically made available when including DLLogger.h.
 * 
 * If you like \c printf, this logger also supports the \c printf() syntax
 * too:
 * \code 
 * dlLogger.dlLog(DL_DEBUG, "this is an int %d, this is a long %f", 4,6.6);
 * int x = 5;
 * dlLogger.dlLog(DL_DEBUG, "and this too is an int: %d", x);
 * \endcode 
 * 
 */
class DLLogger
{
   /// Friend to DLSingletonDestroyer object
   /// to allow the singleton object to be 
   /// destructed at the end of the application.
   template <class SINGLETON> friend class DLSingletonDestroyer;
   private:
   /// Only instance of DLLogger
   static DLLogger* logger;
   /// an instance of DLSingletonDestroyer to 
   /// use to destruct this singleton object
   static DLSingletonDestroyer<DLLogger> _destroyer;

   ///level of logging to incur
   DLSeverity logSeverity ;

   ///logstream for stdout/stderr 
   ostream* logStream;

   ///for output file stream
   FILE* logOutFileStream;

   ///cout, stderr, or file output log type
   DLLogType loggerLogType;
    

   public:

   /**
    * Set the operating severity of the logging system.
    * Only messages of a severity greater than or equal to 
    * \a severity will be output.
    * Messages of a lower severity level will be filtered out.
    * \param severity severity of the log system
    */
   void dlSetLogSeverity(DLSeverity severity);

   /**
    * Sets the logger output type. 
    * 
    * If file output is desired, provide the log file name
    * as the second parameter, \a logFile.
    * \param logType output type of the log system
    * \param logFile output file name
    */
   void dlSetLogType(DLLogType logType, char* logFile = NULL);

   /**
    * Log a C-style message.
    *
    * Example:
    * \code 
    * dlLog(DL_DEBUG, "The Value is %d", 9);
    * \endcode
    *
    * \param severity message severity level
    * \param format output string with optional C-style formatting characters
    * \param ... variable arguments
    */
   void dlLog(DLSeverity severity, const char *format, ...);

   /**
    * Log a \c string message.
    * \param val message
    * \param severity message severity level
    */
   void dlLog(string val, DLSeverity severity);

   /**
    * Log a C-string message.
    * \param val message
    * \param severity message severity level
    */
   void dlLog(const char* val, DLSeverity severity);

   /**
    * Log a single character.
    * \param val \c char value
    * \param severity message severity level
    */
   void dlLog(char val, DLSeverity severity);

   /**
    * Log an integer value.
    * \param val \c int value
    * \param severity message severity level
    */
   void dlLog(int val, DLSeverity severity);

   /**
    * Log a \c float value.
    * \param val \c float value
    * \param severity message severity level
    */
   void dlLog(float val, DLSeverity severity);

   /**
    * Log a \c long integer value.
    * \param val \c long value
    * \param severity message severity level
    */
   void dlLog(long val, DLSeverity severity);

   /**
    * Log a \c short integer value.
    * \param val \c short value
    * \param severity message severity level
    */
   void dlLog(short val, DLSeverity severity);
	
   /**
    * Log a \c double value.
    * \param val \c double value
    * \param severity message severity level
    */
   void dlLog(double val, DLSeverity severity);

   /**
    * Determines if a log message with the given severity level
    * will be output, given the current state of the log
    * severity filter.
    * \param severity severity level to test
    * \return \c true if a message of this severity will be output
    */
   bool isOkToLog(DLSeverity severity);

   /**
    * Returns a handle to the only instance of DLLogger to the user.
    * It will create a new instance if it does not yet exist.
    * \return logger handle
    */
   static DLLogger* dlGetInstance() ;

   /**
    * DLLogger deConstructor 
    */
   virtual ~DLLogger(void);

   private:
   /**
    * The constructor is private because there is only one instance 
    * of DLLogger exist in the DOCLIB.  By having DLLogger's constructor private 
    * will control the creation the DLLogger and ensure only one instance.  
    * The only instance of DLLogger can be created only through the 
    * "dlGetInstance() function.
    */	
   DLLogger(void);


};

/// Global instance of DLLogger.  
static DLLogger &dlLogger =  *DLLogger::dlGetInstance();

#endif // DLLOGGER_H

