#ifndef DLMEMORYBASEDIO_H
#define DLMEMORYBASEDIO_H

#include "DLIOStream.h"

/**
 * \brief DOCLIB memory based I/O Stream 
 * 
 * DLMemoryBasedIO is derived from DLIOStream object and it
 * implements the memory based I/O system for DOCLIB.
 *
 * DLMemoryBasedIO handles the byte order (endian) 
 * for reading/writing int/short in the memory based system.

 * It makes use of the MFS (Memory File System mfs_file.c) to
 * read/write to memory. 
 *
 */

class DLMemoryBasedIO  : public DLIOStream
{

   public:
   /**
    * constructor
    * \param ptr pointer to a memory location to be used
    * for the memory based I/O operations.
    * \param size buffer size 
    * \param mode I/O mode to open as. Same as flags for fopen.
    *  "r" read, "w" write , "rb" read binary, "wb" write binary.
    */
   DLMemoryBasedIO(void *ptr, int size, const char *mode);
   /**
    * Destructor
    */
   virtual ~DLMemoryBasedIO(void);
   /**
    * dlRead reads up to a number of (size) 
    * bytes from the DOCLIB memory I/O stream into the buffer. dlRead will read up
    * to the max size or reached the end of the memory block.
    * The buffer must be allocated prior calling the dlRead() function.
    * The number of bytes read in is returned. 
    *
    * \param buff buffer to store the data
    * \param size max buffer size to read.
    * \return size of bytes read
    */		
   int dlRead(void* buff, int size);
   /**
    * dlGets reads up to a number of (size) 
    * bytes from the DOCLIB memory I/O stream into the buffer. dlGets will read up
    * to the max size or until line feed or reached the end of the memory block.
    * The buffer must be allocated prior calling the dlRead() function.
    * The number of bytes read in is returned. 
    *
    * \param buff buffer to store the data
    * \param size max buffer size to read
    * \return size of bytes read
    */	
   int dlGets(void* buff, int size);

   /**
    * dlGetLine reads a line from the 
    * DOCLIB memory I/O stream. dlGetLine will read up to an end of line or 
    * DL_PPM_MAX_LINE_LENGTH (256) or reached the end of the memory block. A string of
    * the read in line will be returned.
    *
    * \return string that has been read in
    */
   string dlGetLine();

   /**
    * dlGetInt reads an int "sizeof(int) bytes"
    * from the DOCLIB memory I/O stream. dlGetInt will convert the byte order 
    * of the current int value into the indicated byte order.  
    * The read in "int" value is returned. 
    *
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>
    * \return read in int value
    */	
   int dlGetInt(DLEndian endian);
   /**
    * dlGetShort reads a short "sizeof(short) bytes"
    * from the DOCLIB I/O stream. dlGetShort will convert the byte order 
    * of the current short value into the indicated byte order.  
    * The read in "short" value is returned. 
    *
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>
    * \return read in short value 
    */		
   short dlGetShort(DLEndian endian);

   /**
    * dlWriteInt writes an int to 
    * the DOCLIB I/O stream.  dlWriteInt will convert the 
    * int value into the indicated byte order and then writes 
    * it out to the DOCLIB I/O stream.
    *
    * \param intVal int value to write
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>
    */	
   void dlWriteInt(int intVal, DLEndian endian);
   /**
    * dlWriteInt writes a short to 
    * the DOCLIB I/O stream.  dlWriteInt will convert the 
    * short value into the indicated byte order and then writes 
    * it out to the DOCLIB I/O stream.
    *
    * \param shortVal short value to write
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>

   */	
   void dlWriteShort(short shortVal, DLEndian endian);
   /**
    * dlWrite writes a buffer with
    * size to the DOCLIB memory I/O stream.
    * \param buff buffer to write
    * \param size buffer size to write
    * \return size 
    */	
   int dlWrite(void* buff, int size);  
   /**
    * dlClose is a  function to close the memory stream 
    * \return status 
    **/
   int dlClose ();
     
   /**
    * dlFlush flush the current DOCLIB memory I/O stream. 
    *    
    * \return returns 1 if successfully closed the stream.  Otherwise, returns 0 
    */	
   int dlFlush();

   /**
    * dlGetMemBlock () returns the memory 
    * address location where DLMemoryBasedIO is using for I/O operations. 
    * \return memory location
    */
   void* dlGetMemBlock();
   /**
    * dlGetMemSize returns the size of the memory block
    * 
    * \return size of the memory block
    **/
   int dlGetMemSize();

   private:

   /*
    * File descriptor into the memory block.
    * This descriptor is used to identify the different memory 
    * block for different instance of DLMemoryBasedIO.
    */
   int fd;
};

#endif // DLMEMORYBASEDIO_H
