#ifndef _DLPBMIMAGE_H_
#define _DLPBMIMAGE_H_

#include "DLImage.h"
#include "DLBaseImage.h"
#include "DLIOStream.h"
#include "DLSingletonDestroyer.h"
/**
 * \brief PBM image I/O class.
 *
 * Implements loading and saving for the PBM image format, and is used by DLImageFactory. 
 * See DLImage for the typical interface for loading and saving images. 
 */
class DLPBMImage : public DLBaseImage
{
   /// Friend to DLSingletonDestroyer object
   /// to allow the singleton object to be 
   /// destructed at the end of the application.
   template <class SINGLETON> friend class DLSingletonDestroyer;
   private:
   /**
    * Default DLPBMImage Constructor
    */
   DLPBMImage();

   ///Instance of DLPBMImage
   static DLPBMImage* pbmImageReader;
   /// an instance of DLSingletonDestroyer to 
   /// use to destruct this singleton object
   static DLSingletonDestroyer<DLPBMImage> _destroyer;

   protected:
   /**
    * Default DLPBMImage Destructor
    */
   virtual ~DLPBMImage();

   /**
    * dlStream is a private IO object that handles the reads
    * and writes.  PBM uses this object to read/write the 
    * image output source
    * \param dlStream Doclib IO stream for read/write
    * \return dlimage
    */
   DLImage dlLoadImage(DLIOStream &dlStream);

   /**
    * save a dlimage to the dlStream
    * \param dlStream Doclib IO stream for read/write
    * \param dlimage DLImage object to be saved
    */
   void dlSaveImage(DLIOStream &dlStream, const DLImage& dlimage);

   public:
   /// \return A handle to the DLPBMImage object
   static DLPBMImage * createImageReader();


   /**
    * Load PBM Image
    * \param[in] filename name of file to be loaded	
    * \param[out] dlimage DLImage object passed by reference;
    * will be overwritten to contain image from PBM file.
    */
   void dlLoadImage(const char* filename, DLImage& dlimage);

   /**
    * Load PBM Image
    * \param filename name of PBM image file to be opened
    * \return DLImage from file
    */
   DLImage dlLoadImage(const char* filename);

   /**
    * Load PBM Image from memory
    * \param  pMemLoc pointer to beginning of memory buffer
    * \param nSize size of memory buffer in bytes
    * \return DLImage from buffer
    */
   DLImage dlLoadImage(const void* pMemLoc, int nSize);

   /**
    * Loads a particular page of a multiple page
    * image into a DLImage
    * \param filename name of image file to be opened
    * \param pageNumber number of page to be opened
    * \return DLImage from particular page
    */
   DLImage dlLoadImageAtPage(const char* filename, int pageNumber);

   /**
    * Loads a particular page of a multiple page
    * image from an image buffer in memory.
    * \param pMemLoc pointer to beginning of memory image buffer to be opened
    * \param nSize size of buffer in bytes
    * \param pageNumber number of page to be opened
    * \return DLImage from particular page
    */
   DLImage dlLoadImageAtPage(const void* pMemLoc, int nSize, int pageNumber);

   /**
    * Save PBM Image
    * \param filename name of file to be saved
    * \param dlimage PBM Image to be saved
    */
   void dlSaveImage(const char* filename, const DLImage& dlimage);

   /**
    * Save PBM Image to a memory buffer.  See documentation on DLImage for example code.
    * \param[out] nSize size of the buffer to which the image was saved
    * \param[in] dlimage PBM Image to be saved
    * \return pointer to the beginning of the image buffer
    */
   void* dlSaveImageToMem(int& nSize, const DLImage &dlimage);

   /**
    * Determines if the image file is in the PBM format
    * and is supported for loading by DLPBMImage.
    * \param filename name of file to check
    * \return boolean: true if supported
    **/
   bool dlSupportImageFile(const char* filename);

   /**
    * Determines if the memory buffer is in the PBM format
    * and is supported for loading by DLPBMImage.
    * \param pMemLoc pointer to beginning of buffer to check
    * \param size size of buffer in bytes
    * \return boolean: true if supported
    **/
   bool dlSupportImageBuffer(const void* pMemLoc, int size);

   /**
    * Get the total number of pages in the image
    * \param  pMemLoc pointer to beginning of memory buffer
    * \param nSize buffer size in bytes
    * \return number of pages
    */
   int dlGetTotalPages(void* pMemLoc, int nSize);

   /**
    * Get the total number of pages in the image
    * \param  filename file name
    * \return number of pages
    */
   int dlGetTotalPages(const char* filename);
};//end class

static DLPBMImage * dlPbmProcessor = DLPBMImage::createImageReader();

#endif //_DLPBMIMAGE_H_


