#ifndef _DLPNGIMAGE_H_
#define _DLPNGIMAGE_H_

#include "DLImage.h"
#include "DLBaseImage.h"
#include "DLConst.h"
#include "png.h"
#include "DLSingletonDestroyer.h"
/** \cond **/
void dlPNGWarning ( png_structp png_ptr, png_const_charp warning_msg );
/** \endcond **/

/**
 * \brief PNG image I/O class.
 *
 * Implements loading and saving for the PNG image format, and is used by DLImageFactory.
 * See DLImage for the typical interface for loading and saving images.
 */
class DLPNGImage : public DLBaseImage
{
   /// Friend to DLSingletonDestroyer object
   /// to allow the singleton object to be 
   /// destructed at the end of the application.
   template <class SINGLETON> friend class DLSingletonDestroyer;
   private:
   /**
    * Default DLPNGImage Constructor
    */
   DLPNGImage();
   
   ///Only instance of PNGImage
   static DLPNGImage* pngImageReader;
   /// an instance of DLSingletonDestroyer to 
   /// use to destruct this singleton object
   static DLSingletonDestroyer<DLPNGImage> _destroyer;

   protected:
   /**
    * Default DLPNGImage Destructor
    */
   virtual ~DLPNGImage();
   public:

   /** \cond **/
   static void user_read_data(png_structp png_ptr, png_bytep data, png_size_t length);
   /** \endcond **/

   /**
    * \return A handle to the DLPNGImage object.
    */
   static DLPNGImage * createImageReader();

   /**
    * Load PNG Image
    * \param fp
    * \return DLImage to PNG Image
    */
   DLImage dlLoadImage(FILE* fp);

   /**
    * Load PNG Image
    * \param[in] filename name of file to load
    * \param[out] image DLImage object passed by reference; will be overwritten to contain image from PNG file
    */
   void dlLoadImage(const char* filename, DLImage &image);

   /**
    * Load PNG Image from a memory buffer
    * \param  pMemLoc pointer to beginning of memory buffer
    * \param nSize size of buffer in bytes
    * \return DLImage of buffer
    */
   DLImage dlLoadImage(const void* pMemLoc, int nSize);

   /**
    * Load PNG Image
    * \param filename name of file to be opened
    * \return DLImage of file
    */
   DLImage dlLoadImage(const char* filename);

   /**
    * dlLoadImageAtPage will load a particular page from a multiple page
    * image.
    * \param filename name of file to be opened
    * \param pageNumber number of page to be opened
    * \return DLImage of particular page
    */
   DLImage dlLoadImageAtPage(const char* filename, int pageNumber);

   /**
    * dlLoadImageAtPage will load a particular page from a multiple page
    * image residing in a memory buffer.
    * \param pMemLoc pointer to beginning of memory image buffer to be opened
    * \param nSize size of buffer in bytes
    * \param pageNumber number of page to be opened
    * \return DLImage of particular page in buffer
    */
   DLImage dlLoadImageAtPage(const void* pMemLoc, int nSize, int pageNumber);

/** \cond **/

   /**
    * Process the image data from PNG data structures.
    * This is an internal function used by DLPNGImage.
    * By this point the structs I/O will have been set correctly.
    * \param png_ptr  png_struct* to the png struct
    * \param info_ptr info_struct* to the png_info struct
    */
   DLImage dlProcessImage(png_structp png_ptr , png_infop info_ptr);

/** \endcond **/ 

   /**
    * Save PNG Image
    * \param filename name of file to be saved
    * \param image image to be saved
    */
   void dlSaveImage(const char* filename, const DLImage& image);

   /**
    * \brief Save a PNG image with a particular compression scheme. 
    *
    * Not yet implemented.
    * \param filename pathname for file to be saved
    * \param dlimage image object to be saved
    * \param PNG_COMPRESSION_TYPE png compression type
    */
   static void dlSaveImage(const char* filename, const DLImage& dlimage, int PNG_COMPRESSION_TYPE);

   /**
    * Save PNG Image to memory
    * \param[out] nSize size of tiff file in memory
    * \param[in] dlimage DLImage*  PNG Image
    * \return pointer to the memory buffer in which the image was saved
    * \throws DL_NOT_SUPPORTED_EXCEPTION
    */
   void* dlSaveImageToMem( int& nSize, const DLImage& dlimage);

   /**
    * Determines if an image file is in the PNG format and 
    * is supported for loading by DLPNGImage. 
    * \param filename name of file to be checked
    * \return boolean: true if supported
    **/
   bool dlSupportImageFile(const char* filename);

   /**
    * Determines if an image memory buffer is in the PNG format and 
    * is supported for loading by DLPNGImage.
    * \param pMemLoc pointer to beginning of memory image buffer to be checked
    * \param size buffer size in bytes
    * \return boolean: true if supported
    **/
   bool dlSupportImageBuffer(const void* pMemLoc, int size);

   /**
    * Get the total number of pages in the image
    * \param  pMemLoc pointer to beginning of memory buffer
    * \param nSize buffer size in bytes
    * \return number of pages
    */
   int dlGetTotalPages(void* pMemLoc, int nSize);

   /**
    * Get the total number of pages in the image
    * \param  filename file name
    * \return number of pages
    */
   int dlGetTotalPages(const char* filename);
};//end class

static DLPNGImage * dlPngProcessor = DLPNGImage::createImageReader();

#endif //_DLTIFFIMAGE_H_
