#ifndef _DLPOINT_H_
#define _DLPOINT_H_

#include <iostream>

/// Represents a list of points; used by DLSkew.
struct DLPointList {
   int x, y;
   struct DLPointList *next;
} ;


/**
 * \brief Representation of a point on a DLImage.
 *
 * In a DLImage, the origin, (0,0), is located at the upper-left-hand corner
 * of the image.
 * The <i>x</i>-axis in the horizontal direction on the image, with values 
 * increasing as you go to the right.
 * The <i>y</i>-axis is along the vertical direction on the image, with
 * values increading as you head downwards.
 */
class DLPoint
{

   public:

   /**
    * Default DLPoint Constructor; creates a point at the origin, (0,0).
    */
   DLPoint() { x = 0; y = 0;};

   /**
    *  DLPoint Copy  Constructor
    * \param orig DLPoint to copy
    */
   DLPoint(const DLPoint & orig){*this = orig;};

   /**
    *  Assignment operator
    * \param right DLPoint to copy
    */
   const DLPoint & operator=(const DLPoint & right)
      {
         this->x = right.x;
         this->y = right.y;
         return *this;
      };

   /**
    * Equality operator (compare two points)
    * \param right DLPoint to compare to
    */
   inline bool operator==(const DLPoint & right)
      {
         return (this->x == right.x && this->y == right.y);
      };

   /**
    * Translation operator (vector addition)
    * \param p2 DLPoint to translate by
    */
   const DLPoint & operator+=(const DLPoint & p2)
   {
      (this->x) += p2.x;
      (this->y) += p2.y;
      return *this;
   }

   /**
    * DLPoint destructor
    */
   virtual ~DLPoint() { };

   /// <i>x</i>-coordinate, or column, of the point. 
   int x;                      
   /// <i>y</i>-coordinate, or row, of the point.
   int y;   


   /**
    * Constructor: creates a DLPoint at (\a x1 , \a y1).
    * \param x1 <i>x</i>-coordinate, or column, of the point
    * \param y1 <i>y</i>-coordinate, or row, of the point
    */	
   DLPoint(int x1, int y1) { x = x1; y = y1;};
};

/// The sum of two DLPoint%s.
/// \relates DLPoint
/// \param p1 first point
/// \param p2 second point
/// \return a point which is the vector sum of the two points.
inline DLPoint operator+ (const DLPoint & p1, const DLPoint & p2)
{
   return DLPoint(p1.x+p2.x, p1.y+p2.y);
}


/// Stream output of a DLPoint.
/// Allows for the following:
/// \code
/// DLPoint p(0,0);
/// cout << "the point is: " << p << endl;
/// \endcode
/// \relates DLPoint
/// \param outs the output stream; goes on the left-hand side of the operator
/// \param p the point to output
/// \return a reference to the same stream; allows for chaining of multiple
///         objects in the stream output statement as in the above example.
inline std::ostream & operator<< (std::ostream & outs, const DLPoint & p)
{
   outs << '(' << p.x << ',' << p.y << ')';
   return outs;
}


//// add a new point type, used by degradation class
//struct DLPointDouble {
//	double x;
//	double y;
//};

#endif // _DLPOINT_H_
