#ifndef DLSINGLETONDESTROYER_H
#define DLSINGLETONDESTROYER_H


/**
 * \brief An object used to destroy singleton objects
 * 
 * Singleton objects are long-lived objects that live through 
 * the entire lifespan of an application.  Most of the time, the memory
 * allocated by the singleton object will be released when 
 * the application terminates. However, the destructor of the singleton 
 * object is not explicitly called in order to give the object
 * a graceful destruction, since the singleton objects are 
 * created on the heap.
 *
 * The main purpose of the DLSingletonDestroyer object is to 
 * gracefully destroy a singleton object.  
 *
 * This class should only be used within a singleton object. 
 * Upon creation, a singleton object registers itself with the
 * DLSingletonDestroyer, which then takes responsibility for the
 * object's deletion.
 * In DOCLIB this class is used with DLImageFactory
 * and the various image format classes.
 */


template <class SINGLETON>
class DLSingletonDestroyer {
   public:

   /**
    * Default constructor
    */
   DLSingletonDestroyer(){};

   /**
    * Destructor: deletes the instance of the singleton object 
    * which this object is responsible for.
    */
   ~DLSingletonDestroyer(){
      delete _singleton;
   };

   /** 
    * Assign responsibility for the destruction of a singleton object to this destroyer.
    * \param s pointer to the singleton object
    */
   void SetSingleton(SINGLETON* s){
      _singleton = s;
   };

   private:
   /// pointer to the singleton object
   /// to be destructed.
   SINGLETON* _singleton;
};
#endif //DLSINGLETONDESTROYER_H
