#ifndef _DLSKELETON_H_
#define _DLSKELETON_H_

#include "DLImage.h"

//#define MATRIX_SIZE 9

/**
 * \brief Image skeletonization
 *
 * The DLSkeleton object contains a selection of algorithms that 
 * will process a DLImage object and return back a skeleton
 * of the image. 
 *
 * The algorithms are implemented as static functions; therefore,
 * you do not need to instantiate a DLSkeleton object to call these functions. 
 * A typical function call is as follows:  
 * \code
 * DLImage i("test.tif");
 * DLImage iskel = DLSkeleton::dlExtractSkeletonRutovitz(i);
 * \endcode
 */
class DLSkeleton
{
   private:
   static int _rows;
   static int _cols;
   static unsigned char *_data;
   static int _bufRows;
   static int _bufCols;
   static unsigned char *_bufferData;


   public:
/** \cond **/
   /**
    * Default DLSkeleton Constructor
    */
   DLSkeleton();

   /**
    *  DLPoint Copy  Constructor
    */
   DLSkeleton(const DLSkeleton & orig);

   /**
    *  Assignment operator
    * \param right 
    */
   const DLSkeleton & operator=(const DLSkeleton & right);

   /**
    * Default DLSkeleton Destructor
    */
   virtual ~DLSkeleton();
/** \endcond **/

   /**
    * Extracts the skeleton of an image.
    * "This is a one-subiteration algorithm that uses information from a 4 x 4 window."
    * Can be further explained from the following paper:
    * D. Rutovitz "Pattern Recognition," J. Roy. Stat. Soc., vol. 129, Series A pp. 504-530, 1966.
    *
    * \param imageData an image that is to be converted to a skeletonized image
    * \see DLMedialAxis
    */
   static DLImage dlExtractSkeletonRutovitz(const DLImage & imageData); 

   private:
   static unsigned char * dlSkeletonizeRutovitz( int pixel );
   /*static int dlGetNeighborCount(int p[]);
     static int dlNo01Patterns(int p[] ); */
   static int dlSkelRutovitz(  int passNo );
   static void dlGetNeighbors(int row, int col, int p[], int &neighborCnt,
                              int &patterns);
//	static int dlGetPixel(unsigned char *data, int row, int col, int cols);

};//end class

#endif //_DLSKELETON_H_
