#ifndef _DLSKEW_H_
#define _DLSKEW_H_

#include "DLImage.h"
#include "DLPoint.h"
#include "DLBitsPerPixelConverter.h"
#include "DLAngle.h"

#include <cmath>

/**
 * \brief Skew detection algorithm.
 *
 * The algorithm is implemented as a static function, so no object of
 * class DLSkew needs to be instantiated. See below for a usage example.
 */
class DLSkew
{
   private:
   typedef struct {
      float x,y,area;
   } DLSkewSpot;

   public:
   /// \cond
   /**
    * Default DLSkew Constructor
    */
   DLSkew();

   /**
    * Default DLSkew Destructor
    */
   virtual ~DLSkew();
   /// \endcond

   /**
    * Detect the skew angle of an image using the method from TwoBit
    * <i>(elaboration needed)</i>.
    * This feature is not yet complete.
    *
    * \param in input image.
    * \return Best skew angle within the specified range, in degrees. 
    */ 
   static double dlDetectSkewTwoBit(const DLImage &in);

   /**
    * Detect the skew angle of an image using projection method. 
    * \overload
    */
   static double dlDetectSkew(const DLImage & image, const DLAngle & lowerBound,
                              const DLAngle & upperBound, const DLAngle & delta);

   /**
    * Detect the skew angle of an image using projection method -
    * fast algorithm.  
    * The algorithm searches angles in increments given by
    * \c delta in the range 
    * [\c lowerBound , \c upperBound].
    * Positive angles correspond to
    * a clockwise rotation; negative angles are counterclockwise.
    *
    * The following example detects the skew of an image, checking
    * the angles {-30, -29.9, -29.8, ... , +30}:
    * \code
    * DLImage i("test.tif");
    * double ang = DLSkew::dlDetectSkew(i, -30, 30, 0.1)
    * \endcode
    *
    * The algorithm used operates directly on a grayscale image.
    * Therefore, if the input image is not grayscale, 
    * a conversion is automatically performed on a copy of the 
    * input image. The input image
    * itself is not modified.
    * If the input image is binary, 
    * DLBitsPerPixelConverter::dlBinary2Gray() is used to convert 
    * the binary image to a grayscale image.
    * If input image is color, 
    * DLBitsPerPixelConverter::dlDownscaleColor2Binary_threshold() is used to 
    * downscale to binary and then DLBitsPerPixelConverter::dlBinary2Gray()
    * is used to upscale to grayscale. 
    *
    * \param image input image.
    * \param lowerBound the skew angle in degrees where the search starts. Valid angles from -45 to 45 degrees.
    * \param upperBound the skew angle in degrees where the search ends. Valid angles from -45 to 45 degrees.
    * \param delta the step size in angle increment in degrees
    * \return Best skew angle within the specified range, in degrees. 
    * For example, if the image had no skew, but the search bounds were [20,30], a value of 20 is returned. 
    */
   static double dlDetectSkew(const DLImage& image, double lowerBound = -20, double upperBound = 20, double delta = 1);	

   private:

   static double dliGetSkewAngleTwoBit(int iCols, int iRows, DLSkewSpot *points, int number_points, double max_angle);


   /**
    * Calculate the accumulator bins in skew detection algorithm and return the skew angle in radians
    * \param byteImage *DLImage to the grayscale image used in skew detection
    * \param head the *DLPointList to the head of struct DLPointList used in skew detection
    * \param lowerBound the skew angle in degrees where the search starts. Valid angles from -45 to 45 degrees.
    * \param upperBound the skew angle in degrees where the search ends. Valid angles from -45 to 45 degrees.
    * \param delta the step size in angle increment in degrees. Searches only angles indicated by the step size
    * \return double clockwise skew angle in radians 
    */
   static double dlCalculateSkew(DLImage *byteImage, DLPointList *head, double lowerBound, double upperBound, double delta) ;


};//end class

#endif //_DLSKEW_H_
