#ifndef _DLTIFFIMAGE_H_
#define _DLTIFFIMAGE_H_

#include "DLImage.h"
#include "DLBaseImage.h"
#include "DLConst.h"
#include "DLIOStream.h"
#include "DLLogger.h"
#include "DLSingletonDestroyer.h"
#include <cstdarg>

typedef struct tiff TIFF;

void dlTIFFError ( char const *module, char const *fmt, va_list ap);
void dlTIFFWarning ( char const *module, char const *fmt, va_list ap);


/**
 * \brief TIFF image I/O class.
 *
 * Implements loading and saving for the TIFF image format, and is used by
 * DLImageFactory. See DLImage for the typical interface for loading and saving
 * images.
 *
 * Special save functions which are unique to the TIFF format are shown here in 
 * the group of static member functions.
 */
class DLTIFFImage : public DLBaseImage
{
   /// Friend to DLSingletonDestroyer object
   /// to allow the singleton object to be 
   /// destructed at the end of the application.
   template <class SINGLETON> friend class DLSingletonDestroyer;
   private:
   /**
    * Default DLTIFFImage Constructor
    */
   DLTIFFImage();

   /**
    * Load TIFF Image given a TIFF pointer
    * \param image TIFF*
    * \return Loaded TIFF Image
    */
   DLImage dlLoadImage(TIFF* image, int pageNumber = 1);

   ///Only instance of TIFFImage
   static DLTIFFImage* tiffImageReader;
   /// an instance of DLSingletonDestroyer to 
   /// use to destruct this singleton object
   static DLSingletonDestroyer<DLTIFFImage> _destroyer;

   bool dlWangTIFFCheck(const void* pMemLoc, int nSize);
   bool dlWangTIFFCheck(const char* filename);

   static DL_BYTE * dlFlipDataForMinIsWhite( const DLImage &dlimage );

   protected:

   public:
   /**
    * Default DLTIFFImage Destructor
    */
   virtual ~DLTIFFImage();

   /**
    * \return A handle to the DLTIFFImage object
    */
   static DLTIFFImage * createImageReader();

   /**
    * Load TIFF Image
    * \param[in] filename name of file to be loaded
    * \param[out] dlimage  DLImage object passed by reference;
    * will be overwritten to contain image from TIFF file.
    */
   void dlLoadImage(const char* filename, DLImage &dlimage);

   /**
    * Load TIFF Image
    * \param filename name of image file to be opened
    * \return DLImage from file
    */
   DLImage dlLoadImage(const char* filename);

   /**
    * Load TIFF Image from memory
    * \param[in] pMemLoc pointer to beginning of memory buffer of TIFF file
    * \param[in] nSize size of TIFF file in memory, in bytes
    * \param[out] dlimage  DLImage object passed by reference;
    * will be overwritten to contain image from TIFF file.
    */
   void dlLoadImage(const void* pMemLoc, int nSize, DLImage &dlimage);

   /**
    * Load TIFF Image from memory
    * \param pMemLoc pointer to beginning of memory buffer of TIFF file
    * \param nSize size of TIFF file in memory, in bytes
    * \return DLImage of buffer
    */
   DLImage dlLoadImage(const void* pMemLoc, int nSize);

   /**
    * Save TIFF image given a TIFF pointer
    * \param image TIFF*
    * \param dlimage image to be saved
    * \param TIFF_COMPRESSION_TYPE compression type to save with
    * \param MIN_IS_WHITE For grayscale and binary images:
    * \li \c true (the default) saves with the minimum bit or byte value 
    * indicating a white pixel;
    * \li \c false save with the minimum value indicating a black pixel.
    */
   static void dlSaveImage(TIFF* image, const DLImage& dlimage, int TIFF_COMPRESSION_TYPE, 
			   bool MIN_IS_WHITE = true);

   /**
    * Save TIFF Image
    * \param filename name of file to be saved
    * \param dlimage image to be saved
    */
   void dlSaveImage(const char* filename, const DLImage& dlimage);

   /**
    * Save an image with a particular compression scheme. 
    * Here are the compression algorithms supported as of LIB TIFF Version 3.6.0:
    * \arg COMPRESSION_CCITTFAX3 Group 3 compression
    * \arg COMPRESSION_CCITTFAX4 Group 4 compression
    * \arg COMPRESSION_JPEG JPEG compression
    * \arg COMPRESSION_DEFLATE Deflate/ZLIB compression
    * \arg COMPRESSION_NONE No compression
    * \arg COMPRESSION_LZW LZW compression
    *
    * Example:
    * \code
    * DLImage i;
    * // ... 
    * DLTIFFImage::dlSaveImage("output.tif", i, COMPRESSION_LZW);
    * \endcode
    * \param filename name of file to be saved
    * \param dlimage image object to be saved
    * \param TIFF_COMPRESSION_TYPE TIFF compression type
    * \param MIN_IS_WHITE For grayscale and binary images:
    * \li \c true (the default) saves with the minimum bit or byte value 
    * indicating a white pixel;
    * \li \c false save with the minimum value indicating a black pixel.
    */
   static void dlSaveImage(const char* filename, const DLImage& dlimage, int TIFF_COMPRESSION_TYPE, 
			   bool MIN_IS_WHITE = true);

   /**
    * Save an image to a memory buffer with a particular compression scheme.
    * Here are the compression algorithms supported as of LIB TIFF Version 3.6.0
    * \arg COMPRESSION_CCITTFAX3 Group 3 compression
    * \arg COMPRESSION_CCITTFAX4 Group 4 compression
    * \arg COMPRESSION_JPEG JPEG compression
    * \arg COMPRESSION_DEFLATE Deflate/ZLIB compression
    * \arg COMPRESSION_NONE No compression
    * \arg COMPRESSION_LZW LZW compression
    *
    *
    * \param[out] nSize size of TIFF file as saved in memory, in bytes
    * \param[in] dlimage image object to be saved
    * \param[in] TIFF_COMPRESSION_TYPE tiff compression type
    * \param[in] MIN_IS_WHITE For grayscale and binary images:
    * \li \c true (the default) saves with the minimum bit or byte value 
    * indicating a white pixel;
    * \li \c false save with the minimum value indicating a black pixel.
    * \return pointer to memory buffer in which the image was saved
    */
   static void* dlSaveImageToMem(int& nSize, const DLImage &dlimage, int TIFF_COMPRESSION_TYPE, 
				 bool MIN_IS_WHITE = true);

   /**
    * Save TIFF Image to memory
    * \param[out] nSize size of tiff file in memory
    * \param[in] dlimage DLImage*  TIFF Image
    * \return pointer to the memory buffer in which the image was saved
    */
   void* dlSaveImageToMem( int& nSize, const DLImage& dlimage);

   /**
    * Loads a particular page from a multiple page
    * image.
    * \param filename name of image file to be opened
    * \param pageNumber number of page to be opened
    * \return DLImage of particular page
    */
   DLImage dlLoadImageAtPage(const char* filename, int pageNumber);

   /**
    * Loads a particular page from a multiple page
    * image stored in a memory buffer
    * \param pMemLoc pointer to beginning of memory image buffer to be opened
    * \param nSize size of buffer in bytes
    * \param pageNumber number of page to be opened
    * \return DLImage of particular page
    */
   DLImage dlLoadImageAtPage(const void* pMemLoc, int nSize, int pageNumber);

   /**
    * get the total number of pages in a image
    * \param pMemLoc pointer to beginning of TIFF memory buffer
    * \param nSize size of TIFF file in memory, in bytes
    * \return the total number of pages
    */
   int dlGetTotalPages(void* pMemLoc, int nSize);

   /**
    * dlGetTotalPages returns the total number of
    * pages in the document
    *
    * \param filename name of file to be checked
    * \return the total number of pages 
    */
   int dlGetTotalPages(const char* filename);

   /**
    * Determines if an image file is in the TIFF format and
    * is supported for loading by DLTIFFImage.
    * \param filename name of file to be checked
    * \return boolean: true if supported
    **/
   bool dlSupportImageFile(const char* filename);

   /**
    * Determines if an image memory buffer is in the TIFF format and
    * is supported for loading by DLTIFFImage.
    * \param pMemLoc pointer to beginning of memory image buffer to be checked
    * \param size buffer size in bytes
    * \return boolean: true if supported
    **/
   bool dlSupportImageBuffer(const void* pMemLoc, int size);

};//end class

static DLTIFFImage * dlTiffProcessor = DLTIFFImage::createImageReader();

#endif //_DLTIFFIMAGE_H_
