#ifndef _DLTAGLIST_H_
#define _DLTAGLIST_H_

#include <string>
#include <map>
#include "DLException.h"

using namespace std;

/**
 * \brief Metadata tag container object.
 *
 * The DLTagList object is a container which can store tags made up of string key-value pairs. 
 * This can be used for storing metadata about images and document components, and
 * is used in DLImageHead::imageTags, DLDocument::documentTags, DLPage::pageTags,
 * and DLZone::zoneTags.
 */

class DLTagList
{
   public:
   map <string, string> tagMap;

   /**
    * Default DLTagList Constructor
    */
   DLTagList();

   /**
    * Assignment operator
    * \param right Tag list to be copied
    */
   virtual DLTagList& operator=(const DLTagList &right)
      {
         tagMap = right.tagMap;
         return *this;
      };

   /**
    * Get the field value of a specified tag
    * \param tagKey key of the specified tag
    * \return the field value of the tag
    */
   string dlGetTag(string tagKey) const;

   /**
    * Set a tag. If a tag with the specified key already exists
    * in this document, it will not be overwritten unless the optional parameter
    * \a overwriteEnabled is set to \c true.
    * \param tagKey key of the document tag
    * \param tagValue value of the document tag
    * \param overwriteEnabled option for overwriting existing field (default is \c false)
    * \throw DL_Exception DL_UNKNOWN_TAG_EXCEPTION
    */
   void dlSetTag(string tagKey, string tagValue, bool overwriteEnabled = false);

   /**
    * Remove the tag with the specified key from the list of tags
    * \param tagKey key of the tag to remove
    */
   void dlDeleteTag(string tagKey);

   /**
    * Erase all the existing tags
    */
   void dlClearTags() { tagMap.clear(); };

   /**
    * Check whether a document tag with the given key exists
    * \param tagKey key of the document tag
    * \return true if <i><b>a</b> tag exists</i> with that key
    */
   bool dlIsTagSet(string tagKey) const;

   /**
    * Check whether the whole tag list is empty
    * \return \c true if the tag list is empty.
    */
   bool dlIsTagListEmpty()  const { return tagMap.empty(); };
   
   /**
    * Default DLTagList Destructor
    */
   virtual ~DLTagList();

   /** 
    * Iterator which iterates over all the elements of a DLTagList.
    * The first and second operators of an iterator may be used to access
    * the tags and keys, respectively, pointed to by the iterator, <i>e.g.</i>:
    *
    * \code
    * for (DLTagList::iterator ii = t->begin();
    *     ii != t->end(); ii++)
    *  cout << ii->first << " " << ii->second << endl;
    * \endcode
    */
   typedef std::map<string, string>::iterator iterator;

   /// Obtain an iterator to the beginning of the DLTagList.
   iterator begin() { return tagMap.begin(); }

   /// Obtain an iterator pointing just past the end of the DLTagList.
   /// The end() iterator is typically used in a \c for loop for bounds checking.
   /// Warning: dereferencing the end()
   /// iterator is undefined, and will likely cause your program to crash.
   iterator end() { return tagMap.end(); }

   /** 
    * Const Iterator which iterates over all the elements of a DLTagList.
    * The first and second operators of an iterator may be used to access
    * the tags and keys, respectively, pointed to by the iterator, <i>e.g.</i>:
    *
    * \code
    * for (DLTagList::const_iterator ii = t->begin();
    *     ii != t->end(); ii++)
    *  cout << ii->first << " " << ii->second << endl;
    * \endcode
    */
   typedef std::map<string, string>::const_iterator const_iterator;

   /// Obtain a const_iterator to the beginning of the DLTagList.
   const_iterator begin() const { return tagMap.begin(); }

   /// Obtain a const_iterator pointing just past the end of the DLTagList.
   /// The end() const_iterator is typically used in a \c for loop for bounds checking.
   /// Warning: dereferencing the end()
   /// iterator is undefined, and will likely cause your program to crash.
   const_iterator end() const { return tagMap.end(); }

   /**
    * Get an iterator pointing to the location of \c tagKey in the map
    * \return iterator
    */
   iterator dlFindTag ( string tagKey ) { return tagMap.find(tagKey); }
  
}; //end class

#endif // _DLTAGLIST_H_
