#ifndef _DLZONE_H_
#define _DLZONE_H_

#include "DLPage.h"
#include "DLTagList.h"
#include "DLImage.h"
#include "DLPoint.h"
#include "DLConst.h"
#include "DLException.h"
#include "DLLogger.h"

#include <list>
#include <string>

using namespace std;

class DLPage;

/**
 * \brief General bounding box container for objects on a document page.
 *
 * DLZone offers a general bounding box concept that allows bounding boxes to have
 * sub-boxes. These child zones are stored in a list structure similar to DLPage
 * and DLDocument, with methods for appending, inserting, and deleting child
 * zones. In addition, DLZone provides methods for merging and splitting zones.
 * Accordingly, a zone and its sub-zones span a dynamic tree structure.
 * Similar to methods in DLPage, DLZone offers access methods for information
 * specific to zones, e.g. width, height etc. A DLZone object is basically defined
 * by its origin, width and height. It allows developers to dynamically change
 * this information. Each DLZone object automatically does a consistency check
 * whenever the developer tries to do so, and denies the operation when it would
 * lead to an inconsistent state, for instance when a zone would extend past the
 * boundaries of its parent.
 *
 * This example shows DLZone%s used as simple bounding boxes:
 * \code
 * // create some zones to represent lines of text
 * DLZone z1("line1", 50, 50, 1000, 75);
 * DLZone z2 = z1; // copies origin and dimensions of first zone
 * z2.dlSetZoneOrigin(50, 150); // shift second zone's origin
 * \endcode
 *
 * Developers can subclass DLZone and adapt it to their needs. For instance,
 * a typical user-defined subclass of DLZone could be DLCharacter.
 *
 * Programmers can traverse through the child zones on a zone using DLChildZonePtrIterator as below:
 *
 * \code
 * //  ... zonePointer is a DLZone* to a zone already populated with child zones
 * DLZone::DLChildZonePtrIterator childZoneIterator;
 * for (childZoneIterator = zonePointer->begin(); childZoneIterator != zonePointer->end(); childZoneIterator++) {
 *   cout << (*childZoneIterator)->dlGetZoneID() << endl;
 * }
 * \endcode
 *
 * Upon its destruction, the DLZone deletes all its child zones, and notifies 
 * any additional DLZone copies which may refer to it that it is being destroyed.
 *
 * When making a copy of a DLZone, all of its child zones will automatically be copied 
 * over to the new DLZone object.
 * However, this is NOT simply a member-wise copy, 
 * since each of its child zones is a newly created
 * object. This ensures when the destructor of a DLZone copy is invoked, it will not
 * result in accidental destruction of the child zones owned by the original DLZone.
 *
 * It is advisable for programmers to use the \c new operator to dynamically allocate memory
 * space when constructing DLPage%s and DLZone%s. The object destructors of DLDocument, 
 * DLPage, and DLZone will release the allocated memory space of associated objects
 * (i.e. all objects below the current object in the DLDocument hierarchy.) However, 
 * if you use the \c new operator to dynamically create an stand-alone object, you will need
 * to use the \c delete operator to reclaim the memory space it occupies.
 */

class DLZone
{
   friend class DLPage;

   public:

   /** 
    * Iterator which iterates over all the zone's child DLZone%*s.
    * Note that dereferencing this iterator gives a DLZone%*, which
    * must be dereferenced again to give an actual DLZone object reference.
    * (See the code example in the class introduction above.)
    */
   typedef std::list <DLZone*>::iterator DLChildZonePtrIterator;

   /**
    * Obtain an iterator pointing to the first of all the child DLZone%*s
    * \return an iterator pointing to the first of all the child DLZone%*s
    */
   DLChildZonePtrIterator begin() { return childZones.begin(); }

   /**
    * Obtain an iterator that addresses the location succeeding the last DLZone%* in childZones.
    * The end() iterator is typically used in a \c for loop for bounds checking
    * (see the example in the class introduction above). Warning: dereferencing the end()
    * iterator is undefined, and will likely cause your program to crash.
    * \return an iterator that addresses the location succeeding the last DLZone%* in childZones
    */
   DLChildZonePtrIterator end() { return childZones.end(); }

   /**
    * Check whether the given iterator is in the valid range
    * \return true if begin() <= \a zoneIter < end(); returns false otherwise
    */
   bool dlIsZoneIteratorValid 
     (DLChildZonePtrIterator zoneIter)  ;

   /**
    * DLZone Constructor: Leaves bounding box coordinates undefined.
    */
   DLZone();

   /**
    * DLZone Constructor
    * \param x <i>x</i>-coordinate, or column, of the zone's origin (upper-left corner)
    * \param y <i>y</i>-coordinate, or row, of the zone's origin
    * \param w width of the zone
    * \param h height of the zone
    */
   DLZone(int x, int y, int w, int h);

   /**
    * DLZone Constructor
    * \param upperleft DLPoint containing upperleft point of the DLZone
    * \param lowerright DLPoint containing lowerright point of the DLZone
    */
   DLZone(const DLPoint & upperleft, const DLPoint & lowerright);

   /**
    * DLZone Constructor
    * \param zoneID ID tag for this zone
    * \param x <i>x</i>-coordinate, or column, of the zone's origin (upper-left corner)
    * \param y <i>y</i>-coordinate, or row, of the zone's origin
    * \param w width of the zone
    * \param h height of the zone
    */
   DLZone(const string &zoneID, int x, int y, int w, int h);

   /**
    * DLZone Constructor:
    * Contructs a new DLZone as a child zone of the given DLPage and/or DLZone.
    * To associate the new DLZone with a parent DLZone, but not with a parent DLPage,
    * set the \a parentPage parameter to \c NULL.
    * To associate the new DLZone with a parent DLPage, but not with a parent DLZone,
    * set the \a parentZone parameter to \c NULL.
    * If either the parent DLZone or DLPage is destroyed, it will notify the DLZone
    * object to set the appropriate backpointer to \c NULL. 
    * \param parentPage DLPage%* of the parent page
    * \param parentZone DLZone%* of the parent zone
    * \param zoneID ID tag for this zone
    * \param x <i>x</i>-coordinate, or column, of the zone's origin (upper-left corner)
    * \param y <i>y</i>-coordinate, or row, of the zone's origin
    * \param w width of the zone
    * \param h height of the zone
    */
   DLZone(DLPage *parentPage, DLZone *parentZone, const string &zoneID, int x, int y, int w, int h);

   /**
    * DLZone Copy Constructor:
    * All the child DLZone%s will also be copied over to the new DLZone object.
    * The DLZone copy points to the same parent DLPage as the source DLZone, and its parentZone is always set to NULL.
    * A DLZone copy is a real image of the DLZone source. It has identical child zone hierarchy 
    * as the source DLZone, in which each child zone is newly created. As DLZone copy is NOT simply 
    * the default member-wise copy, it will not result in the accidental destruction of child zones of the original DLZone
    * when the destructor of the DLZone copy is invoked.
    */
   DLZone(const DLZone &right);

   /**
    * Default DLZone Destructor
    */
   virtual ~DLZone();

   /**
    * Equality operator.
    * \return true if the bounding box information (origin, width and height)
    * of the two zones is equal. If the DLZone%s are part of a DLDocument / DLPage / DLZone
    * hierarchy, the parent DLZone and parent DLPage must also be equal.
    */
   virtual bool operator == (const DLZone &right) const;

   /**
    * Comparison operator.
    * \return true if the first DLZone begins above the second.
    * If the two DLZone%s begin on the same row of the image, the operator
    * returns true  if the first DLZone begins to the left of the second.
    */
   virtual bool operator < (const DLZone &right) const;

   /**
    * DLZone Copy Assignment operator:
    * All the child DLZone%s will also be copied over to the new DLZone object.
    * The DLZone copy points to the same DLPage as the source DLZone and its parentZone is always set to NULL.
    * A DLZone copy is a real image of the DLZone source. It has identical child zone hierarchy 
    * as the source DLZone, in which each child zone is newly created. As DLZone copy is NOT simply 
    * the default member-wise copy, it will not result in the accidental destruction of child zones of the original DLZone object
    * when the destructor of the DLZone copy is invoked.
    * it is not allowed to reset an existing zone via copy assignment
    */
   virtual DLZone& operator=(const DLZone &right);

   /**
    * Get the unique ID of the zone
    * \return the unique ID of the zone
    */
   string dlGetZoneID() const { return zoneID; };

   /**
    * Set the ID string of the zone
    * \param ID the unique ID of the zone
    */
   void dlSetZoneID(const string &ID) { zoneID = ID; };

   /**
    * Get the width of the zone
    * \return int width of the zone
    */
   int dlGetZoneWidth() const { return width; };

   /**
    * Get the height of the zone
    * \return int height of the zone
    */
   int dlGetZoneHeight() const { return height; };

   /**
    * Get page width
    * \return width of parent page
    */
   int	dlGetPageWidth() const;

   /**
    * Get page height
    * \return height of parent page
    */
   int	dlGetPageHeight() const;

   /**
    * Get the number of direct child zones of this zone;
    * i.e., does not count any of the current DLZone's children's children.
    * \return int number of direct child zones of this zone
    */
   int	dlGetNumChildZones() const  { return static_cast<int>(childZones.size()); };

   /**
    * Check whether the current zone has child zones
    * \return true if zone has child zones
    */
   bool dlHasChildZones() const { return (!childZones.empty()); };

   /**
    * Get the upper left corner of the zone
    */
   DLPoint dlGetZoneOrigin() const { return origin; };

   /**
    * Set the upper left corner of the zone
    * \param x <i>x</i>-coordinate, or column, of zone's origin
    * \param y <i>y</i>-coordinate, or row, of zone's origin
    */
   void dlSetZoneOrigin (int x, int y);

   /**
    * Set the upper left corner of the zone
    * \param origin DLPoint containing
    * <i>x</i>-coordinate, or column, 
    * and <i>y</i>-coordinate, or row, of zone's origin
    */
   void dlSetZoneOrigin (const DLPoint & origin);

   /**
    * Set the width of the zone
    * \param w zone width
    */
   void dlSetZoneWidth(int w);

   /**
    * Set the height of the zone
    * \param h zone height
    */
   void dlSetZoneHeight(int h);

   /**
    * Append a new child zone to this zone at the end of the current list of child zones.
    * If this zone points to a valid DLPage as its parent,
    * the appended zone and all its child zones will
    * be added to the list of zone backpointers on that page, so that if that DLPage
    * is destroyed, the new child zones will be notified as well to set their parent DLPage
    * pointers to \c NULL.
    *
    * The current DLZone then takes responsibility for freeing the memory occupied
    * by \a childZone. Therefore, typically \a childZone should have been allocated
    * on the heap using \c new.
    * Taking the address of a DLZone stack object for appending should be done with
    * care, since the object is automatically destroyed when it goes out of scope,
    * potentially causing a dangling pointer within the parent DLZone.
    * In this situation, consider using dlAppendChildZoneCopy() instead.
    * \param childZone new child zone to be appended
    */
   virtual void dlAppendChildZone(DLZone *childZone);

   /**
    * Append a copy of the given child zone to this zone at the end of the current list of child zones.
    * If the zone points to a valid page, the appended zone and all its child zones will
    * be added to the list of zone backpointers on that page.
    * \param childZone new child zone to be appended
    */
   virtual void dlAppendChildZoneCopy(const DLZone &childZone);

   /**
    * Append a list of new child zones to this zone at the end of the current list.
    * If the zone points to a valid page, the appended zone and all its child zones will
    * be added to the list of zone backpointers on that page.i
    * The current DLZone object takes responsibility for freeing the memory occupied
    * by the DLZone%s in \a childZoneList.
    * \param childZoneList list of new child zones to be appended to the zone at the end of the current list
    */
   virtual void dlAppendChildZoneList(list <DLZone*> childZoneList);

   /**
    * Insert a new child zone to the zone at the specified iterator position of the current list.
    * The current DLZone object takes responsibility for freeing the memory occupied
    * by \a childZone. 
    * See dlInsertChildZoneCopy() for inserting a stack DLZone object as a child zone.
    * \param childIter iterator position in the current child zone list where the new child zone is to be inserted.
    * \param childZone new DLZone to be inserted
    */
   virtual void dlInsertChildZone(DLChildZonePtrIterator childIter, DLZone *childZone);

   /**
    * Insert a new child zone to the zone at the specified position of the current list.
    * The current DLZone object takes responsibility for freeing the memory occupied
    * by \a childZone.
    * See dlInsertChildZoneCopy() for inserting a stack DLZone object as a child zone.
    * \param cursorPosition cursor position in the current child zone list where the new child zone is to be inserted.
    *  \a cursorPosition must be between 0 (beginning of the list) and the size of the child zone (end of the list)
    * \param childZone new DLZone to be inserted
    */
   virtual void dlInsertChildZone(int cursorPosition, DLZone *childZone);

   /**
    * Insert a copy of a child zone to the zone at the specified iterator position of the current list.
    * \param childIter position in the current child zone list where the new child zone is to be inserted.
    * \param childZone new DLZone to be copied and inserted
    */
   virtual void dlInsertChildZoneCopy(DLChildZonePtrIterator childIter, const DLZone &childZone);

   /**
    * Insert a copy of the given child zone to the zone at the specified position of the current list.
    * \param cursorPosition cursor position in the current child zone list where the new child zone is to be inserted.
    * \a cursorPosition must be between 0 (beginning of the list) and the size of the child zone (end of the list)
    * \param childZone new DLZone to be copied and inserted
    */
   virtual void dlInsertChildZoneCopy(int cursorPosition, const DLZone &childZone);

   /**
    * Insert a list of new child zones to the zone starting from the specified iterator position in the current list.
    * \param childIter iterator position in the current child zone list where the new child zones are to be inserted
    * \param childZoneList list of new DLZones to be inserted at the specified cursor position in the current list
    */
   virtual void dlInsertChildZoneList(DLChildZonePtrIterator childIter, list <DLZone*> childZoneList);

   /**
    * Insert a list of new child zones to the zone starting from the specified position in the current list.
    * The current DLZone object takes responsibility for freeing the memory occupied
    * by the DLZone%s in \a childZoneList.
    * \param cursorPosition position in the current child zone list where the new child zones are to be inserted.
    *  \a cursorPosition must be between 0 (beginning of the list) and the size of the child zone (end of the list).
    * \param childZoneList list of new DLZones to be inserted at the specified cursor position in the current list
    */
   virtual void dlInsertChildZoneList(int cursorPosition, list <DLZone*> childZoneList);

   /**
    * Delete a child zone at a given iterator position from the current zone.
    * \param childIter iterator position of the child zone to be removed from the child zone list
    */
   virtual void dlDeleteChildZone(DLChildZonePtrIterator childIter);

   /**
    * Delete a child zone at a given position from the current zone.
    * \param cursorPosition position of the child zone to be removed from the child zone list
    */
   virtual void dlDeleteChildZone(int cursorPosition);

   /**
    * Delete a specific child zone from the zone.
    * The child zone will be removed together with any child zones of its own.
    * \param deletedChildZone child zone to be deleted
    */
   virtual void dlDeleteChildZone(DLZone *deletedChildZone);

   /**
    * Clear the list of child zones of the zone
    */
   virtual void dlClearChildZones();

   /**
    * Construct a new parent zone with the current zone and another zone as its child zones.
    * The new DLZone will have dimensions spanning those of the two children.
    * None of the zones to be merged can have parent zones 
    * and they must not have conflicting parent DLPage pointers.
    * 
    * Warning: The code calling this function is responsible for freeing the
    * memory occupied by the returned DLZone.
    * \param Zone1 another zone to be merged
    * \param newZoneID zoneID for the new parent zone
    * \return newly merged parent zone
    */
   virtual DLZone* dlMergeZones(DLZone *Zone1, const string &newZoneID);

   /**
    * Merge two child zones into a single child zone \a childZone1 and delete \a childZone2.
    * The child zones to be merged may have their own child zone hierarchy.
    * \param[in,out] childZone1 first child zone to be merged
    * \param[in] childZone2 second child zone to be merged
    * \param[in] newZoneID string zoneID for the merged zone
    * \return merged child zone
    */
   virtual void dlMergeChildZones(DLZone *childZone1, DLZone *childZone2, const string &newZoneID);

   /**
    * Split a zone leaf (i.e. without its own child zones) into two neighboring child zones.
    * The current zone will become the parent of the two new DLZone%s.
    * \param offset The horizontal or vertical offset point to split. 
    * For example, if the split is to be into two zones side-by-side, the left
    * zone will be of width \a offset - 1 and the right zone will start at \a offset.
    * \param direction indicating the orientation of the split 
    * - horizontally (side-by-side): \a direction = 1
    * - vertically (one on top of the other): \a direction = 2
    * \param childZoneID1 zoneID for the first (left or upper) child zone after split
    * \param childZoneID2 zoneID for the second (right or lower) child zone after split
    */
   virtual void dlSplitZone(int offset, int direction, const string &childZoneID1, const string &childZoneID2);

   /**
    * Get a copy of the image of the page that the zone is on,
    * if the current zone is associated with a DLPage as its parent.
    * \return a copy of page image
    */
   DLImage dlGetPageImage() const;

   /**
    * Get an image corresponding to the zone  (i.e. a cropped region on the page image), 
    * if the current zone is associated with a DLPage as its parent.
    * \return a copy of zone image
    */
   DLImage dlGetZoneImage() const;

   /**
    * Get the parent zone pointer in DLZone%* (base zone class) form, 
    * which can be used to check the pointer to its parent zone
    * when derived classes of DLZone are present.
    * \return the parent zone pointer 
    */
   DLZone * const dlGetParentZonePointer() const	{	return (DLZone * const)parentZone; };

   /// \name Tag List Functions
   //@{

   /**
    * Get the field value of a specified zone tag
    * \param tagKey key of the specified zone tag
    * \return the field value of a specified zone tag
    */
   string dlGetTag(string tagKey) const;

   /**
    * Set a zone tag. If a tag with the specified key already exists
    * in this zone, it will not be overwritten unless the optional parameter
    * \a overwriteEnabled is set to \c true.  Will throw a DLException if \a overwriteEnabled is 
    * set to \c false and \a tagKey exists.
    * \param tagKey key of the document tag
    * \param tagValue value of the document tag
    * \param overwriteEnabled option for overwriting existing field (default is \c false)
    * \throw DL_Exception DL_UNKNOWN_TAG_EXCEPTION
    */
   void dlSetTag(string tagKey, string tagValue, bool overwriteEnabled = false);

   /**
    * Remove the tag with the specified key from the list of zone tags.
    * \param tagKey key of the zone tag
    */
   void dlDeleteTag(string tagKey);

   /**
    * Erase all the existing zone tags
    */
   void dlClearTags() { zoneTags.tagMap.clear(); };

   /**
    * Check whether a document tag with the given key exists
    * \param tagKey key of the document tag
    * \return true if <i><b>a</b> tag exists</i> with that key
    */
   bool dlIsTagSet(string tagKey) const;

   /**
    * Check whether the whole zone tag list is empty
    */
   bool dlIsTagListEmpty()  const { return zoneTags.tagMap.empty(); };

   /**
    * Get an iterator pointing to the location of \c tagKey in the map
    * \return iterator
    */
   DLTagList::iterator dlFindTag ( string tagKey ) { return zoneTags.tagMap.find(tagKey); }
 
   //@}

   protected:

   string zoneID;

   DLPoint origin;
   int width, height;

   // list of attributes
   DLTagList zoneTags;

   // pointer to its page
   DLPage *zonePage;

   // pointer to parent zone, null if there is none
   DLZone *parentZone;

   // list of direct child zones
   list <DLZone*> childZones;

   // list of child zones whose DLZone *parentZone points to this zone
   // this enables the DLZone to keep track of its immediate child DLZones when its destructor is invoked
   list <DLZone*> childZoneBackPointers;

   /**
    * Create a copy of the current DLZone object and return the pointer of this new copy as precisely DLZone* type
    * This is a utility member function solely used by the copy constructor and assignment operator
    * to ensure that the correct derived type of DLZones is recognized when a copy of the page or parent zone is created.
    * In order to fully support derived classes of DLZone, each derived class of DLZone must have a similar member
    * function that returns a copy of the current object in its correct derived type.
    */
   DLZone* dlClone();

   /**
    * Set the DLPage%* of the current zone and all its child zones without adding them to the page.
    * This function appends this zone and all its child zones to the list of zone 
    * back pointers on the DLPage. This enables this zone to be notified if the DLPage
    * it will refer to as its parent is destroyed.
    * If the DLPage%* of these zones has been set to another existing page, the current zone and all its
    * child zones will be first removed from the back pointer notification list of that page
    * before being added to the new page.
    * \param parentPage pointer to the parent page to be set
    */
   void dlSetPagePointer(DLPage *parentPage);

   /**
    * Check whether each child zone in the hierarchy below the current zone is 
    * contained within the test zone specified by the parameters.
    * \param zOrigin origin of the test zone
    * \param zWidth width of the test zone
    * \param zHeight height of the test zone
    * \return true if all the zones in the current zone's hierarchy 
    * are contained within the test zone
    */
   bool dlCheckZoneBoundaries(DLPoint zOrigin, int zWidth, int zHeight) const;

   /**
    * Check whether each child zone below has the same parent DLPage pointer
    * as the current zone.
    * The parent page pointer of the current zone must be set (i.e. not \c NULL).
    * \return true if all the child zones have the same parent page pointer
    * as the current zone
    */
   bool dlCheckPagePointer() const;

   /**
    * Test whether a zone lies within given page image dimensions.
    * \param pageWidth width of the page image
    * \param pageHeight height of the page image
    * \return true if zone lies within the page image
    */
   bool dlIsWithinPageBoundaries (int pageWidth, int pageHeight) const;

}; //end class

#endif // _DLZONE_H_
